# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap;
use strict;
use POSIX ":sys_wait_h";
use Gtk;
use Data::Dumper;
use netmap::host;
use netmap::subnet;
use netmap::network;

use IO::Handle;

our $version  = '0.2.1';
our @maintabs = ();
our @newhooks = ();
our @defmenu  = (['File/Exit'  => [\&fileexit]],
		 ['Help/About' => [\&aboutbox]],
		 ['Help/License' => [\&license]]);

sub filesave {}
sub fileload {}
sub fileexit {
  my $app = shift || return undef;
  $app->appexit;
}

our $license = <<EOF;
netmap - Perl/GTK Network Scanner

Copyright (C) 2003 Joshua Corbin <jcorbin\@jdweb.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
EOF

our $aboutmessage = <<EOF;
*** NetMap - $version ***

NetMap is a network scanner and discovery tool.

Copyright (c) 2003 Joshua Corbin <jcorbin\@jdweb.com>
EOF

sub license {
  my $app  = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  my $text = new Gtk::Text(undef, undef);
  my $scroll = new Gtk::ScrolledWindow($text->hadj, $text->vadj);
  $text->set_editable(0);
  $text->insert(undef, undef, undef, $license);
  $scroll->add($text);
  $scroll->set_policy(1,1);
  $close->signal_connect('clicked', sub {$win->destroy});
  $win->set_title('NetMap License');
  $win->set_default_size($app->{defsize}->[0]*2/3, $app->{defsize}->[1]*2/3);
  $win->position(1);
  $win->set_transient_for($app->{toplevel});
  $win->set_modal(1);
  $win->vbox->pack_start($scroll, 1, 1, 1);
  $win->action_area->add($close);
  $win->show_all;
  return 1;
}

sub aboutbox {
  my $app  = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  my $text = new Gtk::Text(undef, undef);
  my $scroll = new Gtk::ScrolledWindow($text->hadj, $text->vadj);
  $text->set_editable(0);
  $text->insert(undef, undef, undef, $aboutmessage);
  $scroll->add($text);
  $scroll->set_policy(1,1);
  $close->signal_connect('clicked', sub {$win->destroy});
  $win->set_title('About NetMap');
  $win->set_default_size($app->{defsize}->[0]*2/3, $app->{defsize}->[1]*2/3);
  $win->position(1);
  $win->set_transient_for($app->{toplevel});
  $win->set_modal(1);
  $win->vbox->pack_start($scroll, 1, 1, 1);
  $win->action_area->add($close);
  $win->show_all;
  return 1;
}

sub add_newhook {
  if (ref $_[0] eq 'CODE') {
    push @newhooks, [@_];
    return 1;
  } else {
    return undef;
  }
}

sub smartfork {
  my $app      = shift || return undef;
  my $child    = shift;
  my $callback = shift;
  my %param     = @_;
  return undef if ((ref $child ne 'ARRAY')    || (ref $child->[0] ne 'CODE') ||
		   ((defined $callback) &&
		    ((ref $callback ne 'ARRAY') || (ref $child->[0] ne 'CODE'))));
  my @child = @$child;
  my $child = shift @child;

  my $pid = fork;
  unless (defined $pid) {
    warn "[$$] netmap::smartfork: couldn't fork: $!\n";
    return undef;
  }
  if ($pid == 0) {
    if (defined $param{stdout}) {
      close STDOUT;
      open STDOUT, ">&", $param{stdout} || die "[$$] netmap::smartfork: Couldn't reopen stdout: $!\n";
    }
    if (defined $param{stderr}) {
      close STDERR;
      open STDERR, ">&", $param{stderr} || die "[$$] netmap::smartfork: Couldn't reopen stderr: $!\n";
    }
    if (defined $param{stdin}) {
      close STDIN;
      open STDIN, ">&", $param{stdin}   || die "[$$] netmap::smartfork: Couldn't reopen stdin: $!\n";
    }
    $SIG{TERM} = $SIG{INT} =
      sub {
	print "[$$] Caught SIG TERM/INT, exiting...\n";
	print STDERR "[$$] Caught SIG TERM/INT, exiting...\n";
# 	close STDIN  if (defined $param{stdin});
# 	close STDOUT if (defined $param{stdout});
# 	close STDERR if (defined $param{stderr});
	Gtk->_exit(1);
      };
    Gtk->_exit(&$child(@child));
  } else {
    print "[$$] Forked $pid\n";
    push @{$app->{kids}}, [$pid, $callback];
    $SIG{CHLD} = sub {$app->reaper};
  }
  return $pid;
}

# {nmap}->{scanner}->{nmap}->{asroot}
sub reaper {
  my $app = shift || return undef;
  my $kpid;
  while (($kpid = waitpid(-1, WNOHANG)) > 0) {
    my ($cback) = map {$_->[1]} grep {$_->[0] == $kpid} @{$app->{kids}};
    @{$app->{kids}} = grep {$_->[0] != $kpid} @{$app->{kids}};
    my $eval = $? >> 8;
    my $sig  = $? & 127;
    print "[$$] $kpid exited ($eval)\n";
    if (ref $cback eq 'ARRAY') {
      my @cback = @$cback;
      my $code = shift @cback;
      &$code($app, $kpid, $eval, $sig, @cback);
    }
  }
  $SIG{CHLD} = sub {$app->reaper};
}

sub new {
  my $proto = shift || return undef;
  my $class = (ref $proto) || $proto;
  my %param  = @_;
  my $self =
    {appexit  => [],
     appstart => [],
     confdir  => "$ENV{HOME}/.netmap",
     menu     => [],
     kids     => [],
     maintabs => [@maintabs]};
  bless $self, $class;

  system('mkdir', '-p', $self->{confdir}) unless(-d $self->{confdir});
  unless (-f $self->{confdir} . '/version') {
    my $out;
    open $out, ">$self->{confdir}/version";
    print $out "$version\n";
    close $out;
  }

  unless (-f "$self->{confdir}/apppid") {
    my $out;
    open $out, ">$self->{confdir}/apppid" || return undef;
    print $out "$$\n";
    close $out                    || return undef;
  } else {
    my $in;
    my $oldpid;
    open $in, "$self->{confdir}/apppid"   || return undef;
    chomp($oldpid = <$in>);
    close $in                     || return undef;
    unless (kill 0, $oldpid) {
      unlink "$self->{confdir}/apppid";
      my $out;
      open $out, ">$self->{confdir}/apppid" || return undef;
      print $out "$$\n";
      close $out                    || return undef;
    } else {
      warn "Netmap already running pid: $oldpid\n";
      return undef;
    }
  }

  my ($x, $y) = ($param{winx}, $param{winy});

  $x ||= 800;
  $y ||= 600;
  $self->{defsize} = [$x, $y];

  my $pop =
    sub {
      $self->{_statusbar}->pop($self->{_statid});
      $self->{_statpend}--;
      if ($self->{_statpend} == 0) {
	Gtk->timeout_remove($self->{_stattag});
	delete $self->{_stattag};
      }
      return 1;
    };

  $self->{stattimeout} = 2500;
  $self->{_statusbar}  = new Gtk::Statusbar;
  $self->{_statid}     = $self->{_statusbar}->get_context_id('');
  $self->{_statpend}   = 0;
  $self->{_statpop}    = $pop;
  $self->{_statusbar}->signal_connect
    ('text-pushed', sub {
       $self->{_statpend}++;
       Gtk->timeout_remove($self->{_stattag}) if (defined $self->{_stattag});
       $self->{_stattag} = Gtk->timeout_add ($self->{stattimeout}, $pop);
     });

  $self->{network}     = new netmap::network($self, %param);

  grep {$self->add_menuComponent(@$_)} @defmenu;

  local $_;

  for (@newhooks) {
    my @c = @$_;
    my $code = shift @c;
    push @{$self->{modules}}, &$code($self, @c) if (ref $code eq 'CODE');
  }
  $self->_create_toplevel(%param);

  $SIG{TERM} = $SIG{INT} = sub {$self->appexit};
  $SIG{PIPE} = sub {$self->msg('Caught a SIG PIPE! (That shouldn\'t happen ya know ;-) )')};

  return $self;
}

sub msg {
  my $app = shift || return undef;
  my $msg;
  while ($msg = shift) {
    $app->{_statusbar}->push($app->{_statid}, $msg);
    print "[$$] $msg\n";
  }
  return 1;
}

sub appstart {
  my $app = shift || return undef;
  $app->{toplevel}->show_all;
  $app->msg("Starting...");
  for my $cback (@{$app->{appstart}}) {
    my @c = @$cback;
    my $code = shift @c;
    if (ref $code eq 'CODE') {
      &$code($app, @c);
    }
  }
  $app->msg("Netmap-$version started.");
  Gtk->main;
}

sub appexit {
  my $app = shift || return undef;
  $app->msg("Exiting...");

  kill 15, map {$_->[0]} @{$app->{kids}};

  for my $cback (@{$app->{appexit}}) {
    my @c = @$cback;
    my $code = shift @c;
    if (ref $code eq 'CODE') {
      &$code($app, @c);
    }
  }

  $app->msg("Bye!");
  Gtk->main_quit;
  unlink "$app->{confdir}/apppid";
}

sub register_appstart {
  my $self = shift || return undef;
  my $cback = shift || return undef;
  if (ref $cback eq 'ARRAY') {
    push @{$self->{appstart}}, $cback;
    return 1;
  } else {
    return undef;
  }
}

sub register_appexit {
  my $self  = shift || return undef;
  my $cback = shift || return undef;
  if (ref $cback eq 'ARRAY') {
    push @{$self->{appexit}}, $cback;
    return 1;
  } else {
    return undef;
  }
}

sub add_maintab {
  my $app   = shift || return undef;
  my $label = shift || return undef;
  my $tab   = shift || return undef;
  push @{$app->{maintabs}}, [$label, $tab];
  return 1;
}

sub _create_main_pane {
  my $app = shift || return undef;
  $app->{mainarea} = new Gtk::Notebook;
  my $scroll = new Gtk::ScrolledWindow(undef, undef);
  $scroll->add_with_viewport($app->{mainarea});
  $scroll->set_policy(1,1);
  local $_;
  for (sort {$a->[0] cmp $b->[0]} @{$app->{maintabs}}) {
    my ($name, $_tab) = @$_;
    if (ref $_tab eq 'CODE') {
      $app->{mainarea}->append_page(&$_tab($app), new Gtk::Label($name));
    } elsif (ref $_tab eq 'ARRAY') {
      my @c = @$_tab;
      my $code = shift @c;
      if (ref $code eq 'CODE') {
	$app->{mainarea}->append_page(&$code($app, @c), new Gtk::Label($name));
      }
    }
  }
  return $scroll;
}

sub add_menuComponent {
  my $app = shift || return undef;
  my ($path, $value) = @_;
  if ((ref $value eq 'ARRAY') && (ref $value->[0] eq 'CODE')) {
    my $menu = $app->{menu};
    my @path = split /\//, $path;
    my $cure;
    my $curp;
    while (@path) {
      $curp = shift @path;
      last if ($curp eq '*');
      ($cure) = grep {$_->[0] eq $curp} @$menu;
      if (ref $cure eq 'ARRAY') {
	$menu = $cure->[1];
      } elsif (!defined $cure) {
	my $nextm = [];
	push @$menu, [$curp, $nextm];
	$menu = $nextm;
      } else {
	return undef;
      }
    }
#    if ($curp eq '*') {
#      @$menu = (1, $value);
#    } else {
      @$menu = (0, $value);
#    }
  } else {
    return undef;
  }
  return 1;
}

sub _create_menubar {
  my $app = shift || return undef;
  $app->{_menubar} = new Gtk::MenuBar;
  local $_;
  my ($help) = grep {lc $_->[0] eq 'help'} @{$app->{menu}};
  my @menu   = grep {lc $_->[0] ne 'help'} @{$app->{menu}};
  push @menu, $help;
  for (@menu) {
    my ($name, $_menu) = @$_;
    my $this = $app->_dmenu([$name], $_menu) || next;
    $app->{_menubar}->append($this);
  }
}

sub _dmenu {
  my $app = shift || return undef;
  my ($path, $menu) = @_;
  return undef unless(ref $menu eq 'ARRAY');
  my $ret = new Gtk::MenuItem($path->[-1]);

  if ($menu->[0] == 0) {
    my @c = @{$menu->[1]};
    my $code = shift @c;
    $ret->signal_connect('activate', sub {&$code($app, @c)});
  } elsif (ref $menu eq 'ARRAY') {
    local $_;
    my $menuw = new Gtk::Menu;
    for (@$menu) {
      my ($name, $_menu) = @$_;
      my $this = $app->_dmenu([@$path, $name], $_menu) || next;
      $menuw->append($this);
    }
    $ret->configure(0,1);
    $ret->set_submenu($menuw);
  }
  return $ret;
}

sub _create_toplevel {
  my $app = shift || return undef;
  my %param = @_;
  $app->{toplevel} = new Gtk::Window;

  $app->{toplevel}->set_default_size(@{$app->{defsize}});
  $app->{toplevel}->set_title("NetMap");
  $app->{toplevel}->signal_connect('delete_event', sub {$app->appexit});

  $app->_create_menubar;

  my $main = new Gtk::HPaned;
  $main->add1($app->{nettree});
  $main->add2($app->_create_main_pane(%param));
  $main->set_position($app->{defsize}->[0]/4);

  my $lay = new Gtk::VBox(0);
  $lay->pack_start($app->{_menubar},   0, 0, 0);
  $lay->pack_start($main,              1, 1, 0);
  $lay->pack_start($app->{_statusbar}, 0, 0, 0);

  $app->{toplevel}->add($lay);
  $app->{toplevel}->realize;
  $app->{toplevel}->show_all;
  $app->{toplevel}->position(1);
  return 1;
}

1;
