# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::console;
use strict;
use Gtk;
use IO::Handle;

our $default_blocksize = 1024;
our $default_pause     = 250;

sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my %param = @_;
  return undef if (ref $param{app} ne 'netmap');
  my $self  =
    {_edit     => $param{interactive} || 0,
     app       => $param{app},
     cid       => 0,
     prefix    => $param{prefix}    || '',
     pause     => $param{pause}     || $default_pause,
     blocksize => $param{blocksize} || $default_blocksize,
     pipes     => []};
  delete $param{interactive};
  bless $self, $class;
  $self->_build(%param) || return undef;
  return $self;
}

sub attach {
  my $self   = shift || return undef;
  my $pipe   = shift || return undef;
  my %param   = @_;
  my $prefix = $param{prefix} || $self->{prefix};
  my $id     = $self->{cid} = $self->{cid} + 1;
  my $ioid;

  my $dispatch =
    sub {
      my ($fd, $flags) = @_;
      if ($flags->{read}) {
	$self->_ihandle($id, $prefix, $pipe, $ioid);
      }
    };

  if (defined $param{cback}) {
    $self->{_cback}->{$id} = $param{cback};
  }
  $ioid = Gtk::Gdk->input_add($pipe->fileno, ['read'], $dispatch);
  push @{$self->{pipes}}, [$id, $prefix, $pipe, $ioid];
}

sub _ihandle {
  my $self = shift || return undef;
  my ($id, $prefix, $pipe, $ioid) = @_;
  local $_;
  my $c = $pipe->sysread($_, $self->{blocksize});
  if ($c != 0) {
    my $nl = /\n$/ ? 1 : 0;
    my @lines = split /\n/, $_;
    local $_;
    if (defined $self->{_last}->{$id}) {
      $lines[0] = $self->{_last}->{$id} . $lines[0];
    }
    $self->{_last}->{$id} = pop @lines unless($nl);
    for (@lines) {
      chomp;
      $self->_doline($_, $id, $prefix, $pipe, $ioid);
    }
  } elsif ($c == 0) {
    my $tag;
    $tag = Gtk->timeout_add
      ($self->{pause}, sub {
	 $self->_cleanup($id, $prefix, $pipe, $ioid);
	 Gtk->timeout_remove($tag);
       });
  }
}

sub _doline {
  my $self = shift || return undef;
  local $_ = shift;
  my ($id, $prefix, $pipe, $ioid) = @_;
  my $line = ((defined $prefix) ? $prefix : $self->{prefix}) . $_ . "\n";
  $self->{text}->insert($self->{font}, $self->{fg}, $self->{bg}, $line);
}

sub _cleanup {
  my $self = shift || return undef;
  my ($id, $prefix, $pipe, $ioid) = @_;
  Gtk::Gdk->input_remove($ioid);
  close $pipe;

  $self->_doline($self->{_last}->{$id}, $id, $prefix, $pipe, $ioid)
    if (defined $self->{_last}->{$id});

  @{$self->{pipes}} = grep {$_->[0] != $id} @{$self->{pipes}};
  if (defined $self->{_cback}->{$id}) {
    my $cback = $self->{_cback}->{$id};
    delete $self->{_cback}->{$id};
    if (ref $cback eq 'CODE') {
      &$cback($self);
    } elsif (ref $cback eq 'ARRAY') {
      my @c = @$cback;
      my $code = shift @c;
      if (ref $code eq 'CODE') {
	&$code(@c);
      }
    }
  }
}

sub DESTROY {
  my $self = shift || return undef;
  $self->_nuke;
}

sub _nuke {
  my $self = shift || return undef;
  unless (defined $self->{_nuked}) {
    $self->{_nuked} = 1;
    my @a = @{$self->{pipes}};
    map {$self->_cleanup(@$_)} @a;
    @{$self->{pipes}} = ();
  }
}

# sub interactive {
#   my $self = shift || return undef;
#   my $ret  = $self->{_edit};
#   if ($#_ > -1) {
#     my $val = shift;
#     if (($val == 0) || ($val == 1)) {
#       $self->{_edit} = $val;
#       $self->{text}->set_editable($self->{_edit});
#     }
#   }
#   return $ret;
# }

sub widget {
  my $self = shift || return undef;
  return $self->{_widget};
}

sub ipipe {
  my $self = shift || return undef;
  if ($self->{_edit}) {
    return $self->{_opipe};
  } else {
    return undef;
  }
}

use Data::Dumper;

sub _build {
  my $self   = shift || return undef;
  my %param = @_;
  my $text   = new Gtk::Text(undef,undef);
  my $scroll = new Gtk::ScrolledWindow($text->hadj, $text->vadj);
  $scroll->add($text);
  $scroll->set_policy(1,1);
  $self->{text}   = $text;
  $self->{scroll} = $self->{_widget} = $scroll;

#   if ($self->{_edit}) {
#     $self->{text}->set_editable(1);
#     my ($r, $w);
#     pipe $r, $w;
#     $w->autoflush(1);
#     ($self->{_opipe}, $self->{_ipipe}) = ($r, $w);
#     $self->{text}->signal_connect
#       ('key-press-event', sub {
# 	 my ($text, $event) = @_;
# 	 local $_ = $event->{string};
# 	 print $w $_;
# 	 $w->syswrite($_);
#        });
#   }
  return 1;
}

sub widget {
  my $self = shift || return undef;
  return $self->{_widget};
}

1;
