# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::host;
use strict;
use XML::Simple;
use Gtk;

our $AUTOLOAD;
our %fields =
  ('ipv4_address' => \&valid_ipv4,
   'hostname'     => 1,
   'os'           => 1);
our @extensions = ();

sub add_def_extensions {
  my $host_key = shift || return undef;
  my @f = @_;
  my $def_hand  = sub {my $r = $_[1]->{$_[2]}; $_[1]->{$_[2]} = $_[3] if (defined $_[3]); $r};
  my $def_ser   = sub {map {$_[2]->{$host_key}->{$_} = $_[1]->{$_} if(defined $_[1]->{$_})} @f};
  my $def_deser = sub {map {$_[1]->{$_} = $_[2]->{$host_key}->{$_} if(defined $_[2]->{$host_key}->{$_})} @f};
  grep {add_extension($_, $host_key, $def_hand, $def_ser, $def_deser)} @f;
}

sub add_extension {
  my ($name, $key, $handler, $ser, $deser, $ctx) = @_;
  if ((defined $key)       && (ref $handler eq 'CODE') &&
      (ref $ser eq 'CODE') && (ref $deser   eq 'CODE') &&
      ((!defined $ctx) || (ref $ctx eq 'ARRAY'))) {
    push @extensions, [$key, $handler, $ser, $deser, $ctx, $name];
    return 1;
  } else {
    return undef;
  }
}

sub new {
  my $proto = shift || return undef;
  my $class = (ref $proto) || $proto;
  my %param  = @_;
  my $self  = {ports => []};
  bless $self, $class;
  local $_;
  for (keys %param) {
    next unless(defined $fields{$_});
    $self->$_($param{$_});
  }
  for (map {$_->[0]} @extensions) {
    $self->{$_} = {};
  }
  return $self;
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, keyattr => undef, rootname => 'host');
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray => ['port'])});
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {};
  local $_;
  for (keys %fields) {
    my $v = $self->$_;
    $data->{$_} = $v if (defined $v);
  }

  for (@{$self->ports}) {
    my ($proto, $port, $name, $_data) = @$_;
    my $this =
      {port     => $port,
       protocol => $proto,
       name     => $name};
    $this->{data} = $_data if (defined $_data);
    push @{$data->{port}}, $this;
  }
  for (@extensions) {
    my @c = ();
    @c = @{$_->[4]} if(defined $_->[4]);
    &{$_->[2]}($self, $self->{$_->[0]}, $data, @c);
  }
  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  (ref $data eq 'HASH') || return undef;
  local $_;
  for (@{$data->{port}}) {
    $self->add_port(%$_);
  }
  delete $data->{port};
  for (keys %$data) {
    $self->$_($data->{$_});
  }
  for (@extensions) {
    my @c = ();
    @c = @{$_->[4]} if(defined $_->[4]);
    &{$_->[3]}($self, $self->{$_->[0]}, $data, @c);
  }
  return 1;
}

sub ports {
  my $self = shift || return undef;
  my @ret = sort {$a->[0] cmp $b->[0] ||
		  $a->[1] <=> $b->[1]} @{$self->{ports}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub has_port {
  my $self = shift || return undef;
  my ($proto, $port) = @_;
  if ((defined $port) && (defined $proto)) {
    if (grep {$_->[0] eq $proto && $_->[1] == $port} @{$self->{ports}}) {
      return 1;
    } else {
      return 0;
    }
  }
  return undef;
}

sub get_port {
  my $self = shift || return undef;
  my ($proto, $port) = @_;
  if ((defined $port) && (defined $proto)) {
    my ($ret) = grep {$_->[0] eq $proto && $_->[1] == $port} @{$self->{ports}};
    if (wantarray) {return @$ret}
    else           {return [@$ret]}
  }
  return undef;
}

sub add_port {
  my $self = shift || return undef;
  my %param = @_;
  if ((defined $param{port}) && (defined $param{protocol})) {
    if (grep {$_->[0] eq $param{protocol} && $_->[1] == $param{port}} @{$self->{ports}}) {
      return undef;
    } else {
      my @this = (lc $param{protocol}, $param{port});
      push @this, lc $param{name};
      push @this, $param{data};
      push @{$self->{ports}}, [@this];
      return 1;
    }
  } else {
    return undef;
  }
}

sub delall_ports {
  my $self = shift || return undef;
  @{$self->{ports}} = ();
  return 1;
}

sub delall_tcpports {
  my $self = shift || return undef;
  @{$self->{ports}} = grep {$_->[0] ne 'tcp'} @{$self->{ports}};
  return 1;
}

sub delall_udpports {
  my $self = shift || return undef;
  @{$self->{ports}} = grep {$_->[0] ne 'tcp'} @{$self->{ports}};
  return 1;
}

sub del_port {
  my $self = shift || return undef;
  my %param = @_;
  if ((defined $param{port}) && (defined $param{protocol})) {
    @{$self->{ports}} =
      grep {!($_->[0] eq $param{protocol} && $_->[1] == $param{port})} @{$self->{ports}};
    return 1;
  } else {
    return undef;
  }
}

sub valid_ipv4 {$_[0] =~ /^\d+(?:\.\d+){3}$/ ? 1 : 0}

sub AUTOLOAD {
  my $self  = shift || return undef;
  my @a     = split '::', $AUTOLOAD;
  my $field = $a[-1];
  return __field($self, $field, @_);
}

sub __field {
  my $self  = shift || return undef;
  my $field = shift || return undef;
  my $value = shift;
  my $ret   = undef;
  if (defined $fields{$field}) {
    $ret = $self->{$field};
    if (defined $value) {
      if (ref $fields{$field} eq 'CODE') {
	if (&{$fields{$field}}($value)) {
	  $self->{$field} = $value;
	} else {
	  $ret = undef;
	}
      } else {
	$self->{$field} = $value;
      }
    }
  } elsif (grep {$_->[5] eq $field} @extensions) {
    local $_;
    ($_) = grep {$_->[5] eq $field} @extensions;
    my @c = ();
    @c = @{$_->[4]} if (defined $_->[4]);
    $ret = &{$_->[1]}($self, $self->{$_->[0]}, $field, $value, @c);
  }
  return $ret;
}

1;
