# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::hostinfo;
use strict;
use netmap;
use netmap::network;

our @extensions = ();
our @ctxmenu = ();
our $imported=0;
our @fields =
  (
   ['Hostname', 'hostname'],
   ['IPv4 Address', 'ipv4_address'],
   ['Operating System', 'os']
  );

sub import {
  return if ($imported);
  netmap::add_newhook(\&new, 'netmap::hostinfo');
  $imported=1;
}

sub new { # Entry from netmap::new()
  my $app = shift || return undef;
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $self = {wins => {}, ctxmenuactive => 0};
  bless $self, $class;
  $self->{app} = $app;
#  $app->add_menuComponent('Settings/NMap/Host Scanning', [\&hostscanconf, $self]);
  $app->{network}->add_updateHook(sub {$self->netUpdate(@_)});
  $app->{hostinfo} = $self;
  $app->add_maintab('Host Info', sub {$self->tab});
}

sub netUpdate {
  my $self = shift || return undef;
  my $app  = shift || return undef;
  my ($type, $title, $obj) = @_;
  if ($type eq 'host') {
    local $_;
    for (@fields) {
      my $field = $_->[1];
      if ($#$_ == 1) {
	$self->{$field}->set_text($obj->$field);
      } elsif (($#$_ == 2) && (ref $_->[2] eq 'CODE')) {
	&{$_->[2]}($self->{$field}, $obj);
      }
    }
    $self->{ports}->clear;
    local $_;
    for ($obj->ports) {
      my ($proto, $port, $name, $data) = @$_;
      $self->{ports}->append(uc $proto, $port, $name)
    }

    for (@extensions) {
      my ($create, $update) = @$_;
      cback($update, $self, $type, $title, $obj);
    }
    $self->{ports}->set_sensitive(1) if ($#{($obj->ports)} > -1);
  } else {
    $self->{ports}->set_sensitive(0);
    $self->{hostname}->set_text('');
    $self->{ipv4_address}->set_text('');
    $self->{os}->set_text('');
    $self->{ports}->clear;
    local $_;
    for (@fields) {
      my $field = $_->[1];
      $self->{$field}->set_text('');
    }
    for (@extensions) {
      my ($create, $update) = @$_;
      cback($update, $self, undef, undef, undef);
    }
  }
}

sub tab {
  my $self = shift || return undef;
  $self->{tab} = new Gtk::VBox(0);

  local $_;
  for (@fields) {
    my ($title, $field) = @$_;
    my $row = new Gtk::HBox(0);
    $self->{$field} = new Gtk::Label('');
    $row->pack_start(new Gtk::Label($title . ':'), 0, 0, 1);
    $row->pack_start($self->{$field},        0, 0, 1);
    $self->{tab}->pack_start($row,           0, 0, 1);
  }
  $self->{tab}->pack_start(new Gtk::HSeparator, 0, 0, 1);
  $self->{tab}->pack_start($self->portlist,     1, 1, 1);

  for (@extensions) {
    my ($create, $update) = @$_;
    my $ext = cback($create, $self) || next;
    my ($wid, $packinfo) = @$ext;
    next unless ((ref $packinfo eq 'ARRAY') && ($#{$packinfo} == 2));
    $self->{tab}->pack_start(new Gtk::HSeparator, 0, 0, 1);
    $self->{tab}->pack_start($wid, @$packinfo);
  }

  return $self->{tab};
}

sub cback {
  my $cback = shift;
  return undef unless (ref $cback eq 'ARRAY');
  my @c = @$cback;
  my $code = shift @c;
  return undef unless (ref $code eq 'CODE');
  return &$code(@c, @_);
}

sub portlist {
  my $self = shift || return undef;
  my $ret;
  ($ret, $self->{ports}) = $self->_portlist;
  return $ret;
}

sub add_ctxMenuEntry {
  my %param = @_;
  return undef if((!defined $param{cback}) || (!defined $param{title}));
  push @ctxmenu, [$param{title}, $param{protocol}, $param{port}, $param{cback}];
}

sub ctxmenu {
  my $self  = shift || return undef;
  my $event = shift || return undef;
  $self->{ctxmenuactive} = 1;
  my $sel   = $self->{ports}->selection;
  return unless (defined $sel);
  my $proto = $self->{ports}->get_text($sel, 0);
  my $port  = $self->{ports}->get_text($sel, 1);
  my $menu  = new Gtk::Menu;
  my $clear = sub {$self->{ctxmenuactive} = 0};

  local $_;
  for (sort {$a->[0] cmp $b->[0]}
       grep {(($_->[1] eq lc $proto) && ((!defined $_->[2]) || ($_->[2] eq $port)))} @ctxmenu) {
    my ($title, $_proto, $_port, $cback) = @$_;
    my @c = @$cback;
    my $code = shift @c;
    next unless (ref $code eq 'CODE');
    my $item = new Gtk::MenuItem($title);
    $item->signal_connect ('activate', sub {&$code($self->{app}, $proto, $port, @c)});
    $menu->append($item);
  }

  $menu->signal_connect('deactivate',       $clear);
  $menu->signal_connect('cancel',           $clear);
  $menu->signal_connect('activate-current', $clear);
  $menu->realize;
  $menu->show_all;
  $menu->popup(undef, undef, $event->{button}, $event->{time}, undef);
  return 1;
}

sub _portlist {
  my $self = shift || return undef;
  $self->{scroll} = new Gtk::ScrolledWindow(undef, undef);
  $self->{ports}  = Gtk::CList->new_with_titles('Protocol', 'Port', 'Service');
  $self->{scroll}->add_with_viewport($self->{ports});
  $self->{scroll}->set_policy(1,1);

  $self->{ports}->signal_connect
    ('button-press-event',
     sub {$self->ctxmenu($_[1]) if (($_[1]->{button} == 3) && (!$self->{ctxmenuactive}))});

  $self->{ports}->set_sensitive(0);
  return ($self->{scroll}, $self->{ports});
}

1;
