# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::network;
use strict;
use Net::IPv4Addr qw(:all);
use XML::Simple;
use Gtk;
use netmap;
use netmap::ctreeHost;
use netmap::ctreeSubnet;

our $ctxmenuactive = 0;
our @ctxmenu = ();

# This whole module should have been written as a subclass of
# netmap::ctreeSubnet, but it wasn't, and much pain has come from it.

sub new {
  my $proto = shift || return undef;
  my $app   = shift || return undef;
  return undef if (ref $app ne 'netmap');
  my $class = (ref $proto) || $proto;
  my $self  = {hostclass => 'netmap::ctreeHost', subnetclass => 'netmap::ctreeSubnet', children => []};
  bless $self, $class;
  $self->{app} = $app;
  $self->_build;
  $app->register_appstart([\&autoload, $self]);
  $app->register_appexit([\&autosave, $self]);
  $self->add_ctxMenuEntry(type => 'host', title => 'Clear Ports',
			  cback => [\&clrports, $self]);
  $app->add_menuComponent('File/Save',       [\&filesave,   $self]);
  $app->add_menuComponent('File/Save as...', [\&filesaveas, $self]);
  $app->add_menuComponent('File/Open...',    [\&fileopen,   $self]);
  return $self;
}

sub filesave {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  if (defined $self->{_last_file}) {
    my $out;
    unless (open $out, ">$self->{_last_file}") {
      warn "Couldn't open $self->{_last_file}: $!\n";
      return;
    }
    print $out $self->serialize;
    close $out;
    $app->msg("Saved $self->{_last_file}.");
  } else {
    filesaveas($app, $self);
  }

  return 1;
}
sub filesaveas {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  return if (defined $self->{sel});
  $self->{sel} = new Gtk::FileSelection('Save as...');
  $self->{sel}->position(1);
  $self->{sel}->set_transient_for($app->{toplevel});
  $self->{sel}->set_modal(1);
  $self->{sel}->ok_button->signal_connect
    ('clicked', sub {
       my $file = $self->{sel}->get_filename;
       my $out;
       $self->{sel}->destroy;
       $self->{_last_file} = $file;
       delete $self->{sel};
       unless (open $out, ">$file") {
	 warn "Couldn't open $file: $!\n";
	 return;
       }
       print $out $self->serialize;
       close $out;
       $app->msg("Saved $file.");
     });
  $self->{sel}->cancel_button->signal_connect('clicked', sub {$self->{sel}->destroy; delete $self->{sel}});
  $self->{sel}->show_all;
  return 1;
}

sub fileopen {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  return if (defined $self->{sel});
  $self->{sel} = new Gtk::FileSelection('Save as...');
  $self->{selt} = [];
  $self->{selt}->[0] = new Gtk::RadioButton('Append to');
  $self->{selt}->[1] = new Gtk::RadioButton('Replace', $self->{selt}->[0]);
  my $h = new Gtk::HBox(0);
  $h->pack_start($self->{selt}->[0], 0, 0, 0);
  $h->pack_start($self->{selt}->[1], 0, 0, 0);
  $h->pack_start(new Gtk::Label('current network contents.'), 0, 0, 0);
  $self->{sel}->main_vbox->pack_start($h, 0, 0, 0);

  $self->{sel}->position(1);
  $self->{sel}->set_transient_for($app->{toplevel});
  $self->{sel}->set_modal(1);
  $self->{sel}->ok_button->signal_connect
    ('clicked', sub {
       local $/;
       my $file = $self->{sel}->get_filename;
       my $in;
       $self->{sel}->destroy;
       $self->delall if ($self->{selt}->[0]->get_active);
       $self->{_last_file} = $file;
       delete $self->{sel};
       delete $self->{selt};
       return unless (-f $file);
       unless (open $in, $file) {
	 warn "Couldn't open $file: $!\n";
	 return;
       }
       my $xml = <$in>;
       close $in;
       $self->deserialize($xml);
       $app->msg("Loaded $file.");
     });
  $self->{sel}->cancel_button->signal_connect
    ('clicked', sub {$self->{sel}->destroy; delete $self->{sel}; delete $self->{selt};});
  $self->{sel}->show_all;
  return 1;
}

sub clrports {
  my $app = shift || return undef;
  my ($type, $title, $obj, $self) = @_;
  $obj->delall_ports;
  $self->_runUpdateHooks;
}

sub _refresh {
  my $self = shift || return undef;
  local $_;
  for (@{$self->{children}}) {
    my $ref = ref $_->[1];
    my $this = undef;
    if ($ref eq $self->{hostclass}) {
      $this = $_->[1]->ipv4_address;
    } elsif ($ref eq $self->{subnetclass}) {
      $this = $_->[1]->number;
    } else {
      $this = undef;
    }
    $_->[0] = $this;
  }
  @{$self->{children}} = grep {defined $_->[0]} @{$self->{children}};
  return 1;
}

sub del {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  my $ref  = ref $obj;
  my $this;
  $self->_refresh;
  if (defined $ref) {
    if ($ref eq $self->{subnetclass}) {
      $this = $obj->number;
    } elsif ($ref eq $self->{hostclass}) {
      $this = $obj->ipv4_address;
    } else {
      return undef;
    }
  }
  delete $obj->{_parent} if ($obj->{_parent} == $self);
  my ($node) = map {$_->[1]->{_widget}} grep {$_->[0] eq $this} @{$self->{children}};
  $self->{tree}->remove($node);
  @{$self->{children}} = grep {$_->[0] ne $this} @{$self->{children}};
}

sub ctxdel {
  my $self = shift || return undef;
  my ($type, $text, $obj) = @{$self->{cursel}};
  if (defined $obj->{_parent}) {
    my $smbhost = $self->{app}->{smb}->findbyip($obj->ipv4_address);
    $obj->{_parent}->del($obj);
    if (defined $smbhost) {
      $smbhost->parent->del($smbhost->name);
    }
  } elsif ($type eq 'smbworkgroup') {
    $self->{app}->{smb}->del($text);
  } else {
    warn "netmap::network::ctxdel: $text ($obj) is parentless, I have no idea how to delete it!\n";
  }
}

sub ctxadd {
  my $self   = shift || return undef;
  my $parent = $self->{cursel}->[2] if (ref $self->{cursel} eq 'ARRAY');
  $parent  ||= $self;

  my $win  = new Gtk::Dialog;

  my $row2 = new Gtk::HBox(0);
  $row2->pack_start(new Gtk::Label('Type:'), 0, 0, 2);
  my $subnet = new Gtk::RadioButton('Subnet');
  my $host   = new Gtk::RadioButton('Host', $subnet);
  $row2->pack_start($subnet, 0, 0, 2);
  $row2->pack_start($host,   0, 0, 2);
  $win->vbox->pack_start($row2, 0, 0, 0);

  my $row3 = new Gtk::HBox(0);
  $row3->pack_start(new Gtk::Label('Network Number / IPv4 Address:'), 0, 0, 2);
  my $entry = new Gtk::Entry;
  $row3->pack_start($entry, 0, 0, 2);
  $win->vbox->pack_start($row3, 0, 0, 0);

  my $ok   = new Gtk::Button('OK');
  my $can  = new Gtk::Button('Cancel');
  $can->signal_connect('clicked', sub {$win->destroy});
  $win->action_area->add($ok);
  $win->action_area->add($can);
  $ok->signal_connect
    ('clicked', sub {
       my $obj;
       my $text = $entry->get_text;
       if ($text =~ /^\s*$/) {
	 $win->destroy;
	 return;
       }
       if ($subnet->get_active) {
	 netmap::ctreeSubnet->new($self->{tree}, $parent, number => $text);
       } elsif ($host->get_active) {
	 netmap::ctreeHost->new($self->{tree}, $parent, ipv4_address => $text);
       }
       $win->destroy;
     });

  $win->set_title('Add...');
  $win->set_policy(0, 0, 1);
  $win->position(1);
  $win->set_transient_for($self->{app}->{toplevel});
  $win->set_modal(1);
  $win->set_focus($entry);
  $win->signal_connect
    ('key-press-event', sub {
       my ($text, $event) = @_;
       if ($event->{keyval} == 65293) {
	 $ok->clicked;
       } elsif ($event->{keyval} == 65307) {
	 $can->clicked;
       }
     });
  $win->show_all;
}

sub autoload {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $in;
  local $/;
  open $in, "$app->{confdir}/autosave.xml" || return undef;
  my $xml = <$in>;
  close $in;
  $self->deserialize($xml);
  $app->msg("Autoloaded network.");
  return 1;
}

sub autosave {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $out;
  open $out, ">$app->{confdir}/autosave.xml" || return undef;
  print $out $self->serialize;
  close $out;
  $app->msg("Autosaved network.");
  return 1;
}

sub delall {
  my $self = shift || return undef;
  grep {$self->del($_->[1])} @{$self->{children}};
}

sub add {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  my $ref  = ref $obj;
  my $this;
  if ($ref eq $self->{subnetclass}) {
    $this = $obj->number;
  } elsif ($ref eq $self->{hostclass}) {
    $this = $obj->ipv4_address;
  } else {
    return undef;
  }
  if (defined $this) {
    $self->_refresh;
    if (grep {$_->[0] eq $this} @{$self->{children}}) {
      return undef;
    } else {
      $obj->{_parent} = $self;
      push @{$self->{children}}, [$this, $obj];
      return 1;
    }
  } else {
    return undef;
  }
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, rootname => 'network', keyattr => undef);
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray =>
				  ['subnet', 'host', 'port', 'master_browser', 'controllers', 'host', 'share', 'workgroup'])});
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {};
  @{$data->{subnet}} = map {$_->serial_data} $self->subnets;
  @{$data->{host}}   = map {$_->serial_data} $self->hosts;
  $data->{smb}       = $self->{app}->{smb}->serial_data;
  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  my @subnet = @{$data->{subnet}} if (ref $data->{subnet} eq 'ARRAY');
  my @host   = @{$data->{host}}   if (ref $data->{host}   eq 'ARRAY');
  for (sort {$a->{number} cmp $b->{number}} @subnet) {
    netmap::ctreeSubnet->new($self->{tree}, $self, deserdata => $_) ||
	warn "netmap::network::deserial_data: unable to create subnet.\n";
  }
  for (sort {ipv4cmp($a->{ipv4}->{address}, $b->{ipv4}->{address})} @host) {
    netmap::ctreeHost->new($self->{tree}, $self, deserdata => $_) ||
	warn "netmap::network::deserial_data: unable to create host.\n";
  }
  $self->{app}->{smb}->deserial_data($data->{smb});
  return 1;
}

sub findsubnetfor {
  my $self = shift || return undef;
  my $ip   = shift || return undef;
  my $it   = undef;
  for my $subnet ($self->subnets) {
    if (ipv4_in_network($subnet->number, $ip)) {
      my $next = $subnet->findsubnetfor($ip);
      $it = (defined $next) ? $next : $subnet;
      last;
   }
  }
  return $it;
}

sub subnets {
  my $self = shift || return undef;
  my @ret  = map {$_->[1]} grep {ref $_->[1] eq $self->{subnetclass}} @{$self->{children}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub hosts {
  my $self = shift || return undef;
  my @ret  = map {$_->[1]} grep {ref $_->[1] eq $self->{hostclass}} @{$self->{children}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

# netmap::network->find('subnet', '10.0.1.0/24')
sub find {
  my $self = shift || return undef;
  my $type = shift || return undef;
  my $name = shift || return undef;
  $self->_refresh;
  return _find($self, $self->{children}, $type, $name);
}

sub _find {
  my $self = shift || return undef;
  my $a    = shift || return undef;
  my $type = shift || return undef;
  my $name = shift || return undef;
  local $_;
  for (@$a) {
    my $ref = ref $_->[1];
    if (($_->[0] eq $name) &&
	((($ref eq $self->{subnetclass}) && ($type eq 'subnet')) ||
	 (($ref eq $self->{hostclass})   && ($type eq 'host')))) {
      return $_->[1];
    } else {
      if (($ref eq $self->{subnetclass}) && (eval{ipv4_in_network($_->[0], $name)})) {
	return _find($self, $_->[1]->{children}, $type, $name);
      }
    }
  }
  return undef;
}

sub add_ctxMenuEntry {
  my $self = shift || return undef;
  my %param = @_;
  return undef if((!defined $param{cback}) || (!defined $param{title}));
  push @ctxmenu, [$param{title}, $param{type}, $param{cback}];
}

sub ishost {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  return (ref $obj eq $self->{hostclass});
}

sub issubnet {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  return (ref $obj eq $self->{subnetclass});
}

sub curtype {
  my $self = shift || return undef;
  (ref $self->{cursel} eq 'ARRAY') || return undef;
  return $self->{cursel}->[0];
}
sub curtitle {
  my $self = shift || return undef;
  (ref $self->{cursel} eq 'ARRAY') || return undef;
  return $self->{cursel}->[1];
}
sub curobj {
  my $self = shift || return undef;
  (ref $self->{cursel} eq 'ARRAY') || return undef;
  return $self->{cursel}->[2];
}

sub ctxmenu {
  my $self  = shift || return undef;
  my $event = shift || return undef;
  $ctxmenuactive = 1;
  my $menu  = new Gtk::Menu;
  my $clear = sub {$ctxmenuactive = 0};

  local $_;
  if (ref $self->{cursel} eq 'ARRAY') {
    my ($ctype, $ctext, $cobj) = @{$self->{cursel}};
    if (!(defined $ctype) || !(defined $cobj)) {
      $ctxmenuactive = 0;
      return 1;
    }
    if (($ctype eq 'host') && ($cobj->{issmb})) {
      for (sort {$a->[0] cmp $b->[0]}
	   grep {(!defined $_->[1]) || ($_->[1] eq 'smbhost')} @ctxmenu) {
	my ($title, $type, $cback) = @$_;
	my @c = @$cback;
	my $code = shift @c;
	next unless (ref $code eq 'CODE');
	my $item = new Gtk::MenuItem($title);
	$item->signal_connect ('activate', sub {&$code($self->{app}, $ctype, $ctext, $cobj, @c)});
	$menu->append($item);
      }
    }
    for (sort {$a->[0] cmp $b->[0]}
	 grep {(!defined $_->[1]) || ($_->[1] eq $ctype)} @ctxmenu) {
      my ($title, $type, $cback) = @$_;
      my @c = @$cback;
      my $code = shift @c;
      next unless (ref $code eq 'CODE');
      my $item = new Gtk::MenuItem($title);
      $item->signal_connect ('activate', sub {&$code($self->{app}, $ctype, $ctext, $cobj, @c)});
      $menu->append($item);
    }
    my $del = new Gtk::MenuItem('Delete');
    $del->signal_connect('activate', sub {$self->ctxdel});
    $menu->append($del);
    if (($ctype ne 'host') && ($ctype ne 'smbworkgroup')) {
      my $add = new Gtk::MenuItem('Add...');
      $add->signal_connect('activate', sub {$self->ctxadd});
      $menu->append($add);
    }
  } else {
    my $add = new Gtk::MenuItem('Add...');
    $add->signal_connect('activate', sub {$self->ctxadd});
    $menu->append($add);
  }

  $menu->signal_connect('deactivate',       $clear);
  $menu->signal_connect('cancel',           $clear);
  $menu->signal_connect('activate-current', $clear);
  $menu->realize;
  $menu->show_all;
  $menu->popup(undef, undef, $event->{button}, $event->{time}, undef);
  return 1;
}

sub add_updateHook {
  my $self = shift || return undef;
  return undef if (ref $_[0] ne 'CODE');
  push @{$self->{updatehooks}}, [@_];
  return 1;
}

sub _runUpdateHooks {
  my $self = shift || return undef;
  my $app  = $self->{app};

  local $_;
  my @sel = (ref $self->{cursel} eq 'ARRAY') ? @{$self->{cursel}} : (undef, undef, undef);

  for (@{$self->{updatehooks}}) {
    my @c = @$_;
    my $code = shift @c;
    if (ref $code eq 'CODE') {
      &$code($app, @sel, @c);
    }
  }
}

sub _select {
  my ($tree, $self, $item) = @_;
  my @info = $tree->get_node_info($item);
  my ($text, $isleaf) = ($info[0], $info[6]);
  my $type = $isleaf ? 'host' : 'subnet';
  my $obj  = $self->find($type, $text);
  unless (defined $obj) {
    if ($text eq 'SMB Network') {
      $self->{cursel} = [undef, $text, undef];
    } else {
      my $row;
      if ($row = $self->{app}->{smb}->find($text)) {
	($type, $obj) = @$row;
	if ($type eq 'smbhost') {
	  my $host = $self->find('host', $obj->ipv4_address);
	  if (defined $host) {
	    $self->{cursel} = ['host', $host->ipv4_address, $host];
	  }
	} elsif ($type eq 'smbworkgroup') {
	  $self->{cursel} = [$type, $text, $obj];
	}
      } else {
	$self->{cursel} = undef;
      }
    }
  } else {
    $self->{cursel} = [$type, $text, $obj];
  }
  $self->_runUpdateHooks;
}

sub _unselect {
  my ($tree, $self, $item) = @_;
  $self->{cursel} = undef;
  $self->_runUpdateHooks;
}

sub _build {
  my $self = shift || return undef;
  $self->{tree}   = Gtk::CTree->new_with_titles(0, 'IP Network');
  $self->{scroll} = new Gtk::ScrolledWindow(undef, undef);
  $self->{scroll}->add_with_viewport($self->{tree});
  $self->{scroll}->set_policy(1,1);
  $self->{app}->{nettree} = $self->{scroll};
  $self->{tree}->signal_connect
    ('button-press-event',
     sub {$self->ctxmenu($_[1]) if (($_[1]->{button} == 3) && (!$ctxmenuactive))});
  $self->{tree}->signal_connect ('tree-select-row',   \&_select,   $self);
  $self->{tree}->signal_connect ('tree-unselect-row', \&_unselect, $self);
  $self->{smbnode} = $self->{tree}->insert
    (undef, undef, ['SMB Network'], 1, undef, undef, undef, undef, 0, 1);
}

1;
