# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap;
use strict;
use IO::Handle;
use Gtk;
use XML::Simple;
use netmap;
use netmap::nmap::nmapConsole;
use netmap::nmap::scanner;

our %hostscanconf =
  (pingtype    => ['B'],
   scantype    => ['T'],
   opts        => ['-O'],
   sourceip    => undef,
   sourceiface => undef,
   portrange   => undef,
   cod         => 1);

our %pingscanconf =
  (pingtype    => ['B'],
   sourceip    => undef,
   sourceiface => undef,
   cod         => 1,
   opts        => []);

our $imported = 0;
sub import {
  unless ($imported) {
    netmap::add_newhook(\&new, 'netmap::nmap');
    netmap::host::add_def_extensions('nmap', 'tcpseqClass', 'tcpseqDifficulty', 'tcptsseq', 'ipidseq');
    push @netmap::hostinfo::fields, ['IPID Sequence',   'ipidseq'];
    push @netmap::hostinfo::fields,
      ['TCP Sequence',    'tcpseq',
       sub {$_[0]->set_text
	      (join(' ', map {ucfirst} split(' ', $_[1]->tcpseqClass)) .
	       ' (' . $_[1]->tcpseqDifficulty .')')}];
    push @netmap::hostinfo::fields, ['TCP TS Sequence', 'tcptsseq'];
    $imported = 1;
  }
}

our $helpmessage = <<EOF;
The NMap module provides the following:
 Ping Scanning
 Port Scanning
 OS Fingerprinting
 TCP/IPID Sequence Analysis

Many of NMap's features require root
privileges; to achieve this, the NMap
module makes use of sudo if it is found
on your system (actually if it finds gnome-sudo,
it will be used instead.)  This requires that
there be any entry in /etc/sudoers that *at least*
allows the user that is running NetMap to run nmap
as root (preferably without a password.)
EOF

sub help {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  my $text = new Gtk::Text(undef, undef);
  my $scroll = new Gtk::ScrolledWindow($text->hadj, $text->vadj);
  $text->set_editable(0);
  $text->insert(undef, undef, undef, $helpmessage);
  $scroll->add($text);
  $scroll->set_policy(1,1);
  $close->signal_connect('clicked', sub {$win->destroy});
  $win->set_title('NMap Help');
  $win->set_default_size($app->{defsize}->[0]*2/3, $app->{defsize}->[1]*2/3);
  $win->position(1);
  $win->set_transient_for($app->{toplevel});
  $win->set_modal(1);
  $win->vbox->pack_start($scroll, 1, 1, 1);
  $win->action_area->add($close);
  $win->show_all;
  return 1;
}

sub new {
  my $app = shift || return undef;
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $self = {};
  bless $self, $class;
  $self->{app} = $app;
  $self->{hostscanconf} = {%hostscanconf};
  $self->{pingscanconf} = {%pingscanconf};
  $self->{decoys}       = [];
  $self->{asroot}       = 0;
  $self->{usedecoys}    = 0;
  $self->{concurscans}  = 3;
  $self->{nmapconsole} = new netmap::nmap::nmapConsole(nmap => $self);
  unless (defined $self->{nmapconsole}) {
    warn "Couldn't create nmapConsole!\n";
    return undef;
  }
  $self->{scanner} = new netmap::nmap::scanner(nmap => $self);
  unless (defined $self->{scanner}) {
    warn "Couldn't create scanner!\n";
    return undef;
  }
  $self->{nmapconsole}->{runcback} = sub {
    $self->{scanner}->doqueue(title => shift, force => 1);
  };

  $app->add_menuComponent('Help/NMap', [\&help, $self]);
  $app->add_menuComponent('Settings/NMap/Host Scanning', [\&hostscanconf, $self]);
  $app->add_menuComponent('Settings/NMap/Ping Scanning', [\&pingscanconf, $self]);
  $app->add_menuComponent('Settings/NMap/Decoys',        [\&decoys,       $self]);
  $app->add_menuComponent('NMap/Import XML',             [\&importxml,    $self]);

#  $app->add_menuComponent('Host/Probe',                  [\&probehost,    $self]);
#  $app->add_menuComponent('Test',                        [\&test,         $self]);
  $app->{network}->add_ctxMenuEntry(type => 'host', title => 'Probe',
				    cback => [\&probehost, $self]);
  $app->{network}->add_ctxMenuEntry(type => 'subnet', title => 'Ping Scan',
				    cback => [\&pingscan, $self]);
  $app->{network}->add_ctxMenuEntry(type => 'subnet', title => 'Probe All',
				    cback => [\&probeall, $self]);
  $app->add_maintab('NMap Console', [\&nmapconsole, $self]);
  $app->register_appstart([\&appstart, $self]);
  $app->register_appexit([\&appexit, $self]);
}

sub nmapconsole {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  return $self->{nmapconsole}->widget;
}

sub appstart {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $file = "$app->{confdir}/nmap-conf.xml";
  if (-f $file) {
    local $/;
    my $in;
    open $in, $file || return undef;
    my $xml = <$in>;
    close $in;
    $self->deserialize($xml);
    $app->msg("Loaded NMap options.");
  }
  return 1;
}

sub appexit {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $xml  = $self->serialize;
  my $out;
  open $out, ">$app->{confdir}/nmap-conf.xml" || return undef;
  print $out $xml;
  close $out;
  $app->msg("Saved NMap options.");
  return 1;
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {};
  $self->{concurscans}  = $self->{nmapconsole}->concur;
  $data->{hostscanconf} = $self->{hostscanconf};
  $data->{pingscanconf} = $self->{pingscanconf};
  $data->{decoys}       = $self->{decoys};
  $data->{usedecoys}    = $self->{usedecoys};
  $data->{asroot}       = $self->{asroot};
  $data->{concurscans}  = $self->{concurscans};
  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  local $_;
  for (keys %{$data->{hostscanconf}}) {
    next if(($_ eq 'scantype') || ($_ eq 'pingtype') || ($_ eq 'opts'));
    delete $data->{hostscanconf}->{$_}
      if ((ref $data->{hostscanconf}->{$_}) || ($data->{hostscanconf}->{$_} =~ /^\s*$/));
  }
  for (keys %{$data->{pingscanconf}}) {
    next if(($_ eq 'scantype') || ($_ eq 'pingtype') || ($_ eq 'opts'));
    delete $data->{pingscanconf}->{$_}
      if ((ref $data->{pingscanconf}->{$_}) || ($data->{pingscanconf}->{$_} =~ /^\s*$/));
  }

  $data->{hostscanconf}->{opts}     = [] unless (ref $data->{hostscanconf}->{opts}     eq 'ARRAY');
  $data->{hostscanconf}->{scantype} = [] unless (ref $data->{hostscanconf}->{scantype} eq 'ARRAY');
  $data->{hostscanconf}->{pingtype} = [] unless (ref $data->{hostscanconf}->{pingtype} eq 'ARRAY');
  $data->{pingscanconf}->{opts}     = [] unless (ref $data->{pingscanconf}->{opts}     eq 'ARRAY');
  $data->{pingscanconf}->{pingtype} = [] unless (ref $data->{pingscanconf}->{pingtype} eq 'ARRAY');

  @{$data->{decoys}} = grep {!ref} @{$data->{decoys}};
  $self->{hostscanconf} = $data->{hostscanconf};
  $self->{pingscanconf} = $data->{pingscanconf};
  $self->{decoys}       = $data->{decoys};
  $self->{usedecoys}    = $data->{usedecoys};
  $self->{asroot}       = $data->{asroot};
  if (defined $data->{concurscans}) {
    $self->{concurscans}  = $data->{concurscans};
    $self->{nmapconsole}->concur($self->{concurscans});
  }
  return 1;
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, keyattr => undef, rootname => 'nmap-conf');
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray => ['scantype', 'pingtype', 'opts', 'decoys'])});
}

use netmap::nmap::scantypes;
use netmap::nmap::pingtypes;
use netmap::nmap::options;
use netmap::nmap::decoys;

sub hostscanconf {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  $close->signal_connect('clicked', sub {$win->destroy});
  my $row = new Gtk::HBox(0);
  $row->pack_start(netmap::nmap::scantypes->new($self) ||
		   new Gtk::Label('Failed to create scantype'), 1, 1, 0);
  $row->pack_start(netmap::nmap::pingtypes->new($self) ||
		   new Gtk::Label('Failed to create pingtype'), 1, 1, 0);
  $win->vbox->pack_start($row, 1, 1, 0);
  $win->vbox->pack_start(netmap::nmap::options->new($self) ||
			 new Gtk::Label('Failed to create options'), 0, 0, 0);
  $win->action_area->add($close);

  $win->set_title('Host Scanning Options');
  $win->set_default_size(400, 400);
  $win->position(1);
#  $win->set_transient_for($self->{app}->{toplevel});
#  $win->set_modal(1);
  $win->signal_connect ('key-press-event', sub {$close->clicked if ($_[1]->{keyval} == 65307)});
  $win->show_all;
}

sub pingscanconf {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  $close->signal_connect('clicked', sub {$win->destroy});
  $win->vbox->pack_start(netmap::nmap::pingtypes->new($self, type => 'pingscan') ||
			 new Gtk::Label('Failed to create pingtype'), 1, 1, 0);
  $win->vbox->pack_start(netmap::nmap::options->new($self, type => 'pingscan') ||
			 new Gtk::Label('Failed to create options'), 0, 0, 0);
  $win->action_area->add($close);

  $win->set_title('Ping Scanning Options');
  $win->set_default_size(400, 300);
  $win->position(1);
#  $win->set_transient_for($self->{app}->{toplevel});
#  $win->set_modal(1);
  $win->signal_connect ('key-press-event', sub {$close->clicked if ($_[1]->{keyval} == 65307)});
  $win->show_all;
}

sub importxml {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $ctx  = {};
  $ctx->{win} = new Gtk::Dialog;
  $ctx->{go}  = new Gtk::Button('Import');
  $ctx->{close} = new Gtk::Button('Close');
  my $h = new Gtk::HBox(1);
  $h->pack_start($ctx->{go},    1, 1, 1);
  $h->pack_start($ctx->{close}, 1, 1, 1);
  $ctx->{win}->action_area->add($h);
  $h = undef;
  $ctx->{win}->vbox->pack_start($self->{scanner}->importer($app, $ctx->{go}, $ctx->{win})
				|| new Gtk::Label('Failed to create importer.'), 1, 1, 0);
  $ctx->{win}->set_title('Import NMap XML Output');
  $ctx->{win}->set_default_size(400, 10);
  $ctx->{win}->position(1);
  $ctx->{win}->set_transient_for($self->{app}->{toplevel});
  $ctx->{win}->set_modal(1);
  $ctx->{win}->show_all;
  $ctx->{close}->signal_connect('clicked', sub {$ctx->{win}->destroy});
}

sub decoys {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  $close->signal_connect('clicked', sub {$win->destroy});
  $win->action_area->add($close);

  $win->vbox->pack_start(netmap::nmap::decoys->new($self) ||
			 new Gtk::Label('Failed to create decoys'), 1, 1, 0);

  $win->set_title('Decoys');
  $win->set_default_size(400, 300);
  $win->position(1);
#  $win->set_transient_for($self->{app}->{toplevel});
#  $win->set_modal(1);
  $win->signal_connect ('key-press-event', sub {$close->clicked if ($_[1]->{keyval} == 65307)});
  $win->show_all;
}

sub pingscan {
  my $app  = shift || return undef;
  my ($type, $title, $obj, $self) = @_;
  return undef if((ref $self ne 'netmap::nmap') ||
		  (!$self->{app}->{network}->issubnet($obj)));
  $self->{scanner}->scan($obj);
}

sub probeall {
  my $app = shift || return undef;
  my ($type, $title, $obj, $self) = @_;
  local $_;
  for ($obj->hosts) {
    probehost($app, 'host', $_->ipv4_address, $_, $self);
  }
  for ($obj->subnets) {
    probeall($app, 'subnet', $_->number, $_, $self);
  }
  return 1;
}

sub probehost {
  my $app = shift || return undef;
  my ($type, $title, $obj, $self) = @_;
  return undef if ((ref $self ne 'netmap::nmap') ||
		   (!$self->{app}->{network}->ishost($obj)));
  $self->{scanner}->scan($obj);
}

1;
