# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap::options;
use strict;
use Data::Dumper;
use Gtk;

our %hostopts =
  ('Fragmentation'          => '-f',
   'OS Detection'           => '-O',
   'Ident Scanning'         => '-I',
   'Fast Scan'              => '-F',
   'Random Host'            => '-iR',
   'Never Use DNS'          => '-n',
   'Always Reverse'         => '-R',
   'Don\'t randomize ports' => '-r',
   'Randomize Hosts'        => '--randomize_hosts');

our %pingopts =
  ('Random Host'            => '-iR',
   'Never Use DNS'          => '-n',
   'Always Reverse'         => '-R',
   'Don\'t randomize ports' => '-r',
   'Randomize Hosts'        => '--randomize_hosts');

our %fields =
  (srcip  => ['Source IP Address', 'sourceip'],
   srcif  => ['Source Interface',  'sourceiface'],
   prange => ['Portrange',         'portrange']);

sub new {
  my $proto = shift || return undef;
  my $nmap  = shift || return undef;
  my %param = @_;
  my $class = ref $proto || $proto;
  my $self = {nmap => $nmap};
  bless $self, $class;
  $self->{lay}    = new Gtk::VBox(0);
  $self->{rowwid} = $param{rowWidth} || 2;
  $self->{type}   = $param{type}     || 'hostscan';
  $self->{fields} = {%fields};

  if ($self->{type} eq 'hostscan') {
    $self->{opts}   = {%hostopts};
    $self->{array}  = $self->{nmap}->{hostscanconf}->{opts};
    $self->{conf}   = $self->{nmap}->{hostscanconf};
  } elsif ($self->{type} eq 'pingscan') {
    $self->{opts}   = {%pingopts};
    $self->{array}  = $self->{nmap}->{pingscanconf}->{opts};
    $self->{conf}   = $self->{nmap}->{pingscanconf};
    delete $self->{fields}->{prange};
  } else {
    return undef;
  }

  $self->_build || return undef;
  return $self->{lay};
}

# sourceip    => undef,
# sourceiface => undef,
# specports   => 0,
# portrange   => undef,


sub _build {
  my $self = shift || return undef;
  my $row = new Gtk::HBox(0);
  my $c=0;
  for my $opt (sort keys %{$self->{opts}}) {
    $self->{ctx}->{$opt} = new Gtk::CheckButton($opt);
    $self->{ctx}->{$opt}->active(1)
      if (isElemOf($self->{opts}->{$opt}, @{$self->{array}}));
    $self->{ctx}->{$opt}->signal_connect ('toggled', sub {$self->update($opt)});
    $row->pack_start($self->{ctx}->{$opt}, 0, 1, 0);
    if ($c == $self->{rowwid}) {
      $c = 0;
      $self->{lay}->pack_start($row, 0, 0, 0);
      $row = new Gtk::HBox(0);
    } else {
      $c++;
    }
  }
  $self->{lay}->pack_start($row, 0, 0, 0) if($c<=3);

  for my $field (sort keys %{$self->{fields}}) {
    my ($title, $nfield) = @{$self->{fields}->{$field}};
    my $row = new Gtk::HBox(0);
    $self->{ctx}->{'b' . $field} = new Gtk::CheckButton($title);
    $self->{ctx}->{$field}       = new Gtk::Entry;
    $self->{ctx}->{'b' . $field}->set_active(defined $self->{conf}->{$nfield});
    $self->{ctx}->{$field}->set_text($self->{conf}->{$nfield}) if(defined $self->{conf}->{$nfield});

    my $up = sub{$self->updatefield($field, $nfield)};
    $self->{ctx}->{'b' . $field}->signal_connect('toggled', $up);
    $self->{ctx}->{$field}->signal_connect('focus-out-event', $up);

    $row->pack_start($self->{ctx}->{'b' . $field}, 0, 0, 0);
    $row->pack_start($self->{ctx}->{$field},       1, 1, 0);
    $self->{lay}->pack_start($row, 0, 0, 0);
  }

  my $asroot = new Gtk::CheckButton('Run as root');
  $asroot->set_active($self->{nmap}->{asroot});
  $asroot->signal_connect('toggled', sub {$self->{nmap}->{asroot} = $asroot->get_active});
  $self->{lay}->pack_start($asroot, 0, 0, 0);

  return 1;
}

sub updatefield {
  my $self   = shift || return undef;
  my $field  = shift || return undef;
  my $nfield = shift || return undef;
  my $bool   = 'b' . $field;
  if ($self->{ctx}->{$bool}->get_active) {
    # TODO validation
    my $new = $self->{ctx}->{$field}->get_text;
    $self->{conf}->{$nfield} = $new;
    print "Updated $nfield to $new\n";
  } else {
    $self->{conf}->{$nfield} = undef;
  }
}

sub update {
  my $self = shift || return undef;
  my $opt  = shift || return undef;
  my $flag = $self->{opts}->{$opt};
  my $isin = isElemOf($flag, @{$self->{array}});
  my $now  = $self->{ctx}->{$opt}->get_active;
  if ($isin && !$now) {
    @{$self->{array}} = grep {$_ ne $flag} @{$self->{array}};
  } elsif (!$isin && $now) {
    push @{$self->{array}}, $flag;
  }
}

sub isElemOf {
  my $it = shift || return undef;
  local $_;
  for (@_) {
    return 1 if($_ eq $it);
  }
  return 0;
}

sub keybyvalue {
  my ($v, $h) = @_;
  local $_;
  for my $k (keys %$h) {
    return $k if($h->{$k} eq $v);
  }
  return undef;
}

1;

