# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap::scanner;
use strict;
use IO::Handle;
use Gtk;
use File::Temp qw( :mktemp  );
use netmap::nmap;
use netmap::nmap::nmapConsole;
use netmap::nmap::parser;
use netmap::hostinfo;
use netmap::ctreeHost;
use netmap::ctreeSubnet;
use Net::IPv4Addr qw(:all);

our $sudo;
chomp($sudo = `which gnome-sudo`);
unless (-e $sudo) {
  chomp($sudo = `which sudo`);
}
unless (-e $sudo) {
  $sudo = undef;
}
warn "Couldn't find sudo or gnome-sudo on your system, nmap will not be able to run as root!\n" unless (-e $sudo);

sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my %param  = @_;
  my $self  = {nmap => $param{nmap}, nq => 0};
  return undef if (ref $self->{nmap} ne 'netmap::nmap');
  $self->{console} = $self->{nmap}->{nmapconsole};
  $self->{app}     = $self->{nmap}->{app};
  return undef if (ref $self->{console} ne 'netmap::nmap::nmapConsole');
  return undef if (ref $self->{app}     ne 'netmap');
  bless $self, $class;
  my $path = "$self->{app}->{confdir}/tmp";
  system('mkdir', $path) unless (-d $path);
  $self->{tmpl} = $path . '/nmap-XXXXXXXX';
  $self->{console}->{scanner} = $self;
  return $self;
}

sub tmpfile {
  my $self = shift || return undef;
  return mktemp($self->{tmpl}) . '.xml';
}

sub child {
  my ($self, $obj, $tmpfile, $type) = @_;
  my @cmd = ('nmap');
  if (-e $sudo) {
    unshift @cmd, $sudo if ($self->{nmap}->{asroot});
  }
  push @cmd, '-v', '-oX', $tmpfile,;
  if ($type eq 'host') {
    push @cmd, $self->hostargs, $obj->ipv4_address;
  } elsif ($type eq 'subnet') {
    push @cmd, $self->psargs, $obj->number;
  } else {
    return 1;
  }
  print "Executing: @cmd\n\n";
  my ($r, $w);
  pipe $r, $w;
  $w->autoflush(1);
  my $pid = fork;
  if (defined $pid) {
    if ($pid == 0) {
      close STDIN;
      close STDOUT;
      close STDERR;
      close $r;
      open STDOUT, '>&', $w;
      open STDERR, '>&', $w;
      exec(@cmd);
    } else {
      my $oldsig = $SIG{TERM};
      $SIG{TERM} = $SIG{INT} =
	sub {
	  if ($self->{nmap}->{asroot}) {system($sudo, 'kill', $pid)}
	  else                         {kill 15, $pid}
	  &$oldsig();
	};
      local $_;
      while (<$r>) {
	print "$_";
      }
    }
  } else {
    print "Coldn't fork: $!\n";
    return 1;
  }
  return 0;
}

sub hostargs {
  my $self = shift || return undef;
  my $conf = $self->{nmap}->{hostscanconf};
  my @ret =
    (map({"-s$_"} @{$conf->{scantype}}),
    map({"-P$_"} @{$conf->{pingtype}}),
    @{$conf->{opts}});
  push @ret, '-S', $conf->{sourceip}
    if (defined $conf->{sourceip});
  push @ret, '-e', $conf->{sourceif}
    if (defined $conf->{sourceif});
  push @ret, '-p', $conf->{portrange}
    if (defined $conf->{portrange});
  push @ret, '-D', join(',', @{$self->{nmap}->{decoys}})
    if (($self->{nmap}->{usedecoys}) && ($#{$self->{nmap}->{decoys}} > -1));
  return @ret;
}

sub psargs {
  my $self = shift || return undef;
  my $conf = $self->{nmap}->{pingscanconf};
  my @ret = ('-sP', map({"-P$_"} @{$conf->{pingtype}}), @{$conf->{opts}});
  push @ret, '-S', $conf->{sourceip}
    if (defined $conf->{sourceip});
  push @ret, '-e', $conf->{sourceif}
    if (defined $conf->{sourceif});
  push @ret, '-D', join(',', @{$self->{nmap}->{decoys}})
    if (($self->{nmap}->{usedecoys}) && ($#{$self->{nmap}->{decoys}} > -1));
  return @ret;
}

sub importer {
  my $self = shift || return undef;
  my $app  = shift || return undef;
  my $go   = shift || return undef;
  my $win  = shift || return undef;
  my $ctx  = {};
  my $v    = new Gtk::VBox(0);
  my $h    = new Gtk::HBox(0);
  $ctx->{src}    = new Gtk::Entry;
  $ctx->{browse} = new Gtk::Button('Browse');
  $h->pack_start(new Gtk::Label('Source:'), 0, 0, 0);
  $h->pack_start($ctx->{src},               1, 1, 0);
  $h->pack_start($ctx->{browse},            0, 0, 0);
  $v->pack_start($h,                        0, 0, 0);
  $ctx->{ret} = $v;
  $h = undef;
  $v = undef;

  $ctx->{browse}->signal_connect
    ('clicked', sub {
       my $sel = new Gtk::FileSelection('Select File . . .');
       $sel->position(1);
       $sel->set_transient_for($win);
       $sel->set_modal(1);
       $sel->ok_button->signal_connect
	 ('clicked', sub {$ctx->{src}->set_text($sel->get_filename);$sel->destroy});
       $sel->cancel_button->signal_connect('clicked', sub {$sel->destroy});
       $sel->show_all;
     });

  $go->signal_connect
    ('clicked', sub {
       my $src = $ctx->{src}->get_text;
       my $data = netmap::nmap::parser::parsefile($src, mode => 'file');
       for my $host (@$data) {
	 $self->buildhost($self->{app}, undef, $host);
       }
       $win->destroy;
     });

  return $ctx->{ret};
}

sub buildhost {
  my $self = shift || return undef;
  my $app  = shift || return undef;
  my $obj  = shift;
  my $data = shift || return undef;

  if (!defined $obj) {
    $obj = $app->{network}->find('host', $data->{ipv4_address});
    unless (defined $obj) {
      my $parent = $app->{network}->findsubnetfor($data->{ipv4_address});
      unless (defined $parent) {
	$parent = new netmap::ctreeSubnet($app->{network}->{tree},
					  $app->{network},
					  number => scalar ipv4_network($data->{ipv4_address}));
      }
      $obj = new netmap::ctreeHost($app->{network}->{tree}, $parent, ipv4_address => $data->{ipv4_address});
    }
  }

  $obj->hostname($data->{hostnames}->[0]);
  $obj->os(join(' / ', map({$_->[1]} sort {$a->[0] <=> $b->[0]} @{$data->{os}})) || '<Unknown>');

  if (ref $data->{tcpsequence} eq 'HASH') {
    $obj->tcpseqClass($data->{tcpsequence}->{class});
    $obj->tcpseqDifficulty($data->{tcpsequence}->{difficulty});
  }

  $obj->tcptsseq($data->{tcptssequence}->{class})
    if (ref $data->{tcptssequence} eq 'HASH');
  $obj->ipidseq($data->{ipidsequence}->{class})
    if (ref $data->{ipidsequence} eq 'HASH');

  $obj->delall_tcpports
    if(grep {$_->[0] eq 'tcp'} @{$data->{ports}});
  $obj->delall_udpports
    if(grep {$_->[0] eq 'udp'} @{$data->{ports}});
  map {$obj->add_port(protocol => $_->[0],
		      port     => $_->[1],
		      name     => $_->[2])} @{$data->{ports}};
  $app->{network}->_runUpdateHooks
    if($app->{network}->curtitle eq $obj->ipv4_address);
  return 1;
}

sub xml2obj {
  my $self = shift || return undef;
  my %param = @_;
  my $app  = $param{app}  || return undef;
  my $obj  = $param{obj}  || return undef;
  my $type = $param{type} || return undef;
  my $file = $param{file} || return undef;

  if ($type eq 'host') {
    my $data = netmap::nmap::parser::parsefile($file) || return undef;
    $data = $data->[0];
    return undef unless($obj->ipv4_address eq $data->{ipv4_address});

    $self->buildhost($app, $obj, $data) || return undef;

    $app->msg("Host probe on $data->{ipv4_address} complete.");
  } elsif ($type eq 'subnet') {
    my $data = netmap::nmap::parser::parsefile($file, mode => 'subnet') || return undef;
    local $_;
    for (@$data) {
      netmap::ctreeHost->new($obj->{ctree}, $obj, ipv4_address => $_) ||
	  warn "netmap::nmap::scanner::xml2obj: Couldn't add host to subnet.\n";
    }
  } else {
    print STDERR "netmap::nmap::scanner::xml2obj: unknown type\n";
    return undef;
  }
  return 1;
}

sub died {
  my ($app, $pid, $eval, $sig, $self, $obj, $tmpfile, $type) = @_;
  if ($eval == 0) {
    $self->xml2obj(app => $app, obj => $obj, file => $tmpfile, type => $type);
  } else {
    $app->msg("Child exited with error status, scan failed.");
    return undef;
  }
  $self->{nq}--;
  $self->doqueue;
  return 1;
}

sub doqueue {
  my $self  = shift || return undef;
  my %param = @_;
  return undef if (($self->{nq} > $self->{console}->concur) &&
		   (!defined $param{force}) &&
		   ($self->{console}->concur != -1));
  return undef if ($#{$self->{queue}} == -1);
  my $title = $param{title};
  my @jobs = ();
#  my $job;
  if (defined $title) {
#    ($job) = grep {$_->[2] eq $title} @{$self->{queue}};
    push @jobs, grep {$_->[2] eq $title} @{$self->{queue}};
    @{$self->{queue}} = grep {$_->[2] ne $title} @{$self->{queue}};
  } else {
#    $job = shift @{$self->{queue}};
    if ($self->{console}->concur == -1) {
      push @jobs, @{$self->{queue}};
      @{$self->{queue}} = ();
    } else {
      my $c = $self->{console}->concur - $self->{nq};
      for (my $i=0; $i<$c; $i++) {
	push @jobs, shift @{$self->{queue}};
      }
    }
  }

  for my $job (@jobs) {
    next if (ref $job ne 'ARRAY');
    my ($obj, $type, $title) = @$job;
    print "Running $title from queue\n";
    $self->{console}->delfromqueue($title);
    $self->scan($obj, force => 1);
  }


#  $self->doqueue if(($#{$self->{queue}} > -1) &&
#		    (($self->{nq} <= $self->{console}->concur) ||
#		     ($self->{console}->concur == -1)));

  return 1;
}

sub scan {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  my %param = @_;
  my $title;
  my $type;

  if ($self->{app}->{network}->ishost($obj)) {
    $title = $obj->ipv4_address;
    $type = 'host';
  } elsif ($self->{app}->{network}->issubnet($obj)) {
    $title = $obj->number;
    $type = 'subnet';
  } else {
    return undef;
  }

  if (($self->{nq} < $self->{console}->concur) ||
      $param{force} ||
      ($self->{console}->concur == -1)) {
    $self->{nq}++;
    my ($r, $w);
    my $tmpfile = $self->tmpfile;
    pipe $r, $w;
    my $pid = $self->{app}->smartfork([\&child, $self, $obj, $tmpfile, $type],
				      [\&died,  $self, $obj, $tmpfile, $type],
				      stdout => $w,
				      sudokill => $self->{nmap}->{asroot});
    close $w;
    unless (defined $pid) {
      warn "Couldn't smartfork\n";
      return undef;
    }
    unless ($self->{console}->addscanner
	    (pid   => $pid,
	     pipe  => $r,
	     title => $title)) {
      warn "Couldn't add scanner, killing child!\n";
      kill 9, $pid;
      return undef;
    }
  } else {
    $self->{console}->addtoqueue(title => $title);
    push @{$self->{queue}}, [$obj, $type, $title];
  }

  return 1;
}


# sub test {
#   my $app  = shift || return undef;
#   my $self = shift || return undef;
#   my ($r, $w);
#   pipe $r, $w;
#   my $pid = $app->smartfork([\&child], [\&died], stdout => $w);
#   close $w;
#   unless (defined $pid) {
#     warn "Couldn't smartfork\n";
#     return undef;
#   }
#   $self->{_i}++;
#   $self->{nmapconsole}->addscanner
#     (pid   => $pid,
#      pipe  => $r,
#      title => 'test' . $self->{_i})
#       || warn "Couldn't add scanner\n";
# }

# sub child {
#   sleep 1;
#   print "Child started\n";
#   print STDERR "[$$] Child started\n";
#   print "Exiting in:\n";
#   for (my $i=5; $i>0; $i--) {
#     print "  $i\n";
#     sleep 1;
#   }
#   print "bye";
#   print STDERR "[$$] bye";
#   return 0;
# }

1;
