# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::portactions;
use strict;
use Gtk;
use netmap;
use netmap::hostinfo;
use XML::Simple;

our $imported = 0;
our $shell = '/bin/sh';
our $xterm = findprogram('xterm');

sub import {
  unless ($imported) {
    netmap::add_newhook(\&new, 'netmap::portactions');
    $imported = 1;
  }
}

our @defactions =
  (
   ['tcp', '80', 'cmd',   'galeon -n %i', 'Galeon'],
   ['tcp', '*',  'xterm', 'nc %i %p', 'Netcat'],
   ['udp', '*',  'xterm', 'nc -u %i %p', 'Netcat'],
   ['tcp', '80', 'cback', sub {print "hi"}, 'Hi']
  );

sub add_action {
  my $self = shift || return undef;
  my %param = @_;
  push @{$self->{actions}}, [$param{protocol}, $param{port}, $param{type}, $param{cmd}, $param{title}];
}

sub del_action {
  my $self = shift || return undef;
  my %param = @_;
  @{$self->{actions}} = grep {(($_->[0] ne $param{protocol}) ||
			       ($_->[1] ne $param{port})     ||
			       ($_->[2] ne $param{type})     ||
			       ($_->[3] ne $param{cmd}))} @{$self->{actions}};
}

sub new {
  my $app   = shift || return undef;
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $self  = {app => $app, actions => [@defactions]};
  bless $self, $class;
  $app->add_menuComponent('Settings/Port Actions', [sub {$self->conf}]);
  $app->register_appstart([\&appstart, $self]);
  $app->register_appexit([\&appexit, $self]);

  return $self;
}

sub appstart {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $file = "$app->{confdir}/port-actions.xml";
  return unless (-f $file);
  my $in;
  open $in, $file || return undef;
  local $/;
  my $xml = <$in>;
  close $in;
  $self->deserialize($xml);
  $app->msg("Loaded Port Actions");
  local $_;
  for (@{$self->{actions}}) {
    #&$code($self->{app}, $proto, $port, @c)
    my $cback = [\&_cback_entry, $self, [@$_]];
    my $port = $_->[1];
    $port = undef if ($port eq '*');
    push @netmap::hostinfo::ctxmenu, [$_->[4], lc $_->[0], $port, $cback];
  }
}

sub sync {
  my $self = shift || return undef;
  @netmap::hostinfo::ctxmenu = grep {ref $_->[3]->[1] ne 'netmap::portactions'} @netmap::hostinfo::ctxmenu;
  for (@{$self->{actions}}) {
    #&$code($self->{app}, $proto, $port, @c)
    my $cback = [\&_cback_entry, $self, [@$_]];
    my $port = $_->[1];
    $port = undef if ($port eq '*');
    push @netmap::hostinfo::ctxmenu, [$_->[4], lc $_->[0], $port, $cback];
  }
}

sub _cback_entry {
  my ($app, $_proto, $port, $self, $this)  = @_;
  my ($proto, $_port, $type, $cmd, $title) = @$this;
  return if ($app->{network}->curtype ne 'host');
  my $ip = $app->{network}->curtitle;
  # TCP/19: Netcat(xterm) = nc %i %p
  if ($type eq 'cmd') {
    $cmd =~ s/%i/$ip/g;
    $cmd =~ s/%p/$port/g;
    $app->smartfork([sub {$self->docmd($shell, '-c', $cmd)}]);
  } elsif ($type eq 'xterm') {
    $cmd =~ s/%i/$ip/g;
    $cmd =~ s/%p/$port/g;
    $cmd .= "; echo \"Press <Enter> to close\"; read null";
    $app->smartfork([sub {$self->docmd($xterm, '-title', "$title: $ip:$port", '-e', $shell, '-c', $cmd)}]);
  } elsif ($type eq 'cback') {
    if (ref $cmd eq 'ARRAY') {
      my @c = @$cmd;
      my $code = shift @c;
      return unless (ref $code eq 'CODE');
      $app->smartfork([sub {&$code($app, @c)}]);
    } elsif (ref $cmd eq 'CODE') {
      $app->smartfork([sub {&$cmd($app)}]);
    }
  }
}

sub docmd {
  my $self = shift || return undef;
  system(@_);
  return 0;
}

sub appexit {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $file = "$app->{confdir}/port-actions.xml";
  my $out;
  open $out, ">$file" || return undef;
  print $out $self->serialize;
  close $out;
  $app->msg("Saved Port Actions");
}

sub serialize {
  my $self = shift || return undef;
  my $data = $self->serial_data;
  my $xml = XMLout($self->serial_data, keyattr => undef, rootname => 'port-actions');
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray => ['item'])});
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  @{$self->{actions}} = @{$data->{item}};
  @{$self->{actions}} =
    map {my $a = [$_->{protocol}, $_->{port}, $_->{type}, $_->{command}, $_->{title}];$a}
      @{$self->{actions}};
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {item => [ grep( {!ref $_->[3]} @{$self->{actions}}) ]};
  @{$data->{item}} =
    map {my $a = {};
	 $a->{protocol} = $_->[0];
	 $a->{port}     = $_->[1];
	 $a->{type}     = $_->[2];
	 $a->{command}  = $_->[3];
	 $a->{title}    = $_->[4];
	 $a}
      @{$data->{item}};
  return $data;
}

sub conf {
  my $self = shift || return undef;
  $self->{confwin}  = new Gtk::Dialog;
  $self->{close} = new Gtk::Button('Close');
  $self->{lay}   = new Gtk::VBox(0);
  my $row = new Gtk::HBox(0);

  $self->{list}  = Gtk::CList->new_with_titles('Protocol', 'Port', 'Type', 'Command', 'Title');
  $self->{lay}->pack_start($self->{list}, 1, 1, 0);

  $self->{ptcp}  = new Gtk::RadioButton('TCP');
  $self->{pudp}  = new Gtk::RadioButton('UDP', $self->{ptcp});
  $self->{port}  = Gtk::Entry->new_with_max_length (5);
  $self->{xterm} = new Gtk::CheckButton('Run in XTerm');
  $self->{cmd}   = new Gtk::Entry;
  $self->{title} = new Gtk::Entry;
  $self->{add}   = new Gtk::Button('Add');
  $self->{del}   = new Gtk::Button('Del');

  my $destroy = sub {
    local $_;
    for (('confwin', 'close', 'lay', 'list', 'ptcp', 'pudp', 'port', 'xterm', 'cmd', 'add', 'del', 'title')) {
      delete $self->{$_};
    }
  };

  $row->pack_start(new Gtk::Label('Protocol:'), 0, 0, 0);
  $row->pack_start($self->{ptcp},  0, 0, 0);
  $row->pack_start($self->{pudp},  0, 0, 0);
  $row->pack_start(new Gtk::Label('Port:'), 0, 0, 0);
  $row->pack_start($self->{port},  0, 0, 0);
  $row->pack_start($self->{xterm}, 0, 0, 0);
  $self->{lay}->pack_start($row,           0, 0, 0);

  $row = new Gtk::HBox(0);
  $row->pack_start(new Gtk::Label('Title:'), 0, 0, 0);
  $row->pack_start($self->{title},           1, 1, 0);
  $self->{lay}->pack_start($row,             0, 0, 0);

  $row = new Gtk::HBox(0);
  $row->pack_start(new Gtk::Label('Command:'), 0, 0, 0);
  $row->pack_start($self->{cmd},   1, 1, 0);
  $row->pack_start($self->{add},   0, 0, 0);
  $row->pack_start($self->{del},   0, 0, 0);
  $self->{lay}->pack_start($row,           0, 0, 0);

  $row=undef;

  $self->{confwin}->signal_connect('delete_event', sub {&$destroy});
  $self->{close}->signal_connect('clicked',        sub {$self->{confwin}->destroy; &$destroy});

  my ($cursel, $curproto, $curtype, $curcmd, $curport, $curtitle);
  my $clearcur = sub {$cursel = $curproto = $curtype = $curcmd = $curport = $curtitle = undef};
  my $setcur = sub {
    $cursel   = $self->{list}->selection;
    $curtype  = $self->{list}->get_text($cursel, 2);
    $curproto = $self->{list}->get_text($cursel, 0);
    $curport  = $self->{list}->get_text($cursel, 1);
    $curcmd   = $self->{list}->get_text($cursel, 3);
    $curtitle = $self->{list}->get_text($cursel, 4);
  };
  my $resetgui = sub {
    $self->{ptcp}->set_active(1);
    $self->{xterm}->set_active(0);
    $self->{cmd}->set_text('');
    $self->{title}->set_text('');
    $self->{port}->set_text('');
  };
  my $mode = 0;
  my $sync = sub {
    my @new = ();
    for (my $i=0; $i<$self->{list}->rows; $i++) {
      push @new, [lc $self->{list}->get_text($i, 0),
		  $self->{list}->get_text($i, 1),
		  $self->{list}->get_text($i, 2),
		  $self->{list}->get_text($i, 3),
		  $self->{list}->get_text($i, 4)];
    }
    push @new, grep( {ref $_->[3]} @{$self->{actions}} );
    @{$self->{actions}} = @new;
    $self->sync;
  };

  grep {$self->{list}->append(@$_)}
    grep {$_->[0] = uc $_->[0]; $_}
      grep {!ref $_->[3]}
	sort {$a->[0] cmp $b->[0] || $a->[1] <=> $b->[1] || $a->[4] cmp $b->[4]}
	  @{$self->{actions}};

  $self->{add}->signal_connect
    ('clicked', sub {
       my $proto;
       my $type;
       $proto = 'TCP' if ($self->{ptcp}->get_active);
       $proto = 'UDP' if ($self->{pudp}->get_active);
       $type  = $self->{xterm}->get_active ? 'xterm' : 'cmd';
       if ($mode == 0) {
	 $self->{list}->append($proto, $self->{port}->get_text, $type,
			       $self->{cmd}->get_text, $self->{title}->get_text);
       } elsif ($mode == 1) {
	 $self->{list}->set_text($cursel, 0, $proto);
	 $self->{list}->set_text($cursel, 1, $self->{port}->get_text);
	 $self->{list}->set_text($cursel, 2, $type);
	 $self->{list}->set_text($cursel, 3, $self->{cmd}->get_text);
	 $self->{list}->set_text($cursel, 4, $self->{title}->get_text);
	 $self->{list}->unselect_all;
       }
       &$sync();
       &$resetgui();
     });
  $self->{del}->signal_connect
    ('clicked', sub {
       $self->{list}->remove($cursel) if (defined $cursel);
       &$sync();
     });
  $self->{list}->signal_connect
    ('select-row', sub {
       &$setcur();
       if ($curtype eq 'cback') {&$resetgui();return}
       $self->{xterm}->set_active( ((lc $curtype eq 'xterm') ? 1 : 0) );
       $self->{ptcp}->set_active(1) if (lc $curproto eq 'tcp');
       $self->{pudp}->set_active(1) if (lc $curproto eq 'udp');
       $self->{port}->set_text($curport);
       $self->{cmd}->set_text($curcmd);
       $self->{title}->set_text($curtitle);
       $self->{del}->set_sensitive(1);
       $self->{add}->label('Update');
       $mode = 1;
     });
  $self->{list}->signal_connect
     ('unselect-row', sub {
	&$clearcur();
	&$resetgui;
	$self->{del}->set_sensitive(0);
	$self->{add}->label('Add');
	$mode = 0;
      });
  $self->{del}->set_sensitive(0);

  $self->{confwin}->vbox->pack_start($self->{lay}, 1, 1, 0);
  $self->{confwin}->action_area->add($self->{close});
  $self->{confwin}->set_title('Port Actions');
  $self->{confwin}->set_default_size(400, 300);
  $self->{confwin}->position(1);
  $self->{confwin}->signal_connect ('key-press-event', sub {$self->{close}->clicked if ($_[1]->{keyval} == 65307)});
  $self->{confwin}->show_all;
}

sub findprogram {my $ret; chomp($ret = `which $_[0]`); return (-x $ret) ? $ret : undef}

1;
