# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::smb;
use strict;
use Gtk;
use netmap;
use netmap::smb::workgroup;
use netmap::smb::host;
use XML::Simple;

our $imported = 0;
sub import {
  unless ($imported) {
    netmap::add_newhook(\&new, 'netmap::smb');
    $imported = 1;
    push @netmap::hostinfo::extensions, [[\&_create_shares], [\&_update_shares]];
  }
}

sub _create_shares {
  my $hostinfo = shift || return undef;
  my $self = $hostinfo->{app}->{smb}; 
  my $ret = new Gtk::VBox(0);
  my $h2 = new Gtk::HBox(0);

  my $scroll = new Gtk::ScrolledWindow(undef,undef);
  $self->{shares} = Gtk::CList->new_with_titles('Name', 'Type', 'Comment');
  $scroll->add_with_viewport($self->{shares});
  $scroll->set_policy(1,1);

  $self->{username} = new Gtk::Entry;
  $self->{password} = new Gtk::Entry;
  $self->{enumshares} = new Gtk::Button('Enumerate Shares');
  $self->{enumshares}->signal_connect('clicked', sub {$self->enumshares});

  $self->{username}->signal_connect
    ('focus-out-event', sub {
       $self->{curhost}->username($self->{username}->get_text) if(defined $self->{curhost});
     });
  $self->{password}->signal_connect
    ('focus-out-event', sub {
       $self->{curhost}->password($self->{password}->get_text) if(defined $self->{curhost});
     });

  $h2->pack_start(new Gtk::Label('Username:'), 0, 0, 0);
  $h2->pack_start($self->{username},           0, 0, 0);
  $h2->pack_start(new Gtk::Label('Password:'), 0, 0, 0);
  $h2->pack_start($self->{password},           0, 0, 1);

  $self->{cont} = new Gtk::Frame('SMB');
  $self->{cont}->add($scroll);
  $ret->pack_start($self->{cont},       1, 1, 1);
  $ret->pack_start($h2,                 0, 0, 0);
  $ret->pack_start($self->{enumshares}, 0, 0, 0);

  $self->{shares}->set_sensitive(0);
  $self->{enumshares}->set_sensitive(0);
  $self->{username}->set_sensitive(0);
  $self->{password}->set_sensitive(0);

  return [$ret, [1, 1, 1]];
}

sub enumshares {
  my $self = shift || return undef;
  return unless (defined $self->{curhost});
  $self->{curhost}->scan_shares;
  $self->{shares}->clear;
  local $_;
  my @a;
  my @s = $self->{curhost}->shares;
  for (sort {$a->[0] cmp $b->[0] ||
	       $a->[1] cmp $b->[1]}
       map {@a=@$_; @a[0] = $netmap::smb::host::share_types[$a[0]]; [@a]}
       @s) {
    my $type = $_->[0];
    $type =~ s/_SHARE$//;
    $type = ucfirst lc $type;
    $self->{shares}->append($_->[1], $type, $_->[2]);
  }
}

sub _update_shares {
  my $hostinfo = shift || return undef;
  my $self = $hostinfo->{app}->{smb};
  my ($type, $title, $obj) = @_;
  my $host;
  if ((defined $obj) && ($host = $self->findbyip($obj->ipv4_address))) {
    my @flags;
    push @flags, 'LM'  if ($host->master_browser);
    push @flags, 'DMB' if ($host->domain_master_browser);
    push @flags, 'DC'  if ($host->domain_controller);
    my $flags = ($#flags == -1) ? '' : '(' . join(', ', @flags) . ')';
    $self->{cont}->set_label('SMB Shares on ' . sprintf("\\\\%s\\%s %s", $host->domain, $host->name, $flags));
    local $_;
    my @a;
    my @s = $host->shares;
    for (sort {$a->[0] cmp $b->[0] ||
		 $a->[1] cmp $b->[1]}
	 map {@a=@$_; @a[0] = $netmap::smb::host::share_types[$a[0]]; [@a]}
	 @s) {
      my $type = $_->[0];
      $type =~ s/_SHARE$//;
      $type = ucfirst lc $type;
      $self->{shares}->append($_->[1], $type, $_->[2]);
    }
    $self->{username}->set_text($host->username);
    $self->{password}->set_text($host->password);
    $self->{curhost} = $host;
    $self->{shares}->set_sensitive(1);
    $self->{enumshares}->set_sensitive(1);
    $self->{username}->set_sensitive(1);
    $self->{password}->set_sensitive(1);
  } else {
    $self->{shares}->set_sensitive(0);
    $self->{enumshares}->set_sensitive(0);
    $self->{username}->set_sensitive(0);
    $self->{password}->set_sensitive(0);
    $self->{curhost} = undef;
    $self->{shares}->clear;
    $self->{cont}->set_label('SMB');
  }
  return 1;
}

sub new {
  my $app   = shift || return undef;
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $self  = {app => $app, children => []};
  bless $self, $class;
  $app->add_menuComponent('Help/SMB', [\&help, $self]);
  $app->{network}->add_ctxMenuEntry(type => 'subnet', title => 'NetBIOS Scan',
				     cback => [\&scansubnet, $self]);
  $app->{network}->add_ctxMenuEntry(type => 'host',   title => 'NetBIOS Query',
				     cback => [\&queryhost, $self]);
  $app->{network}->add_ctxMenuEntry(type => 'smbhost',   title => 'Enumerate Shares',
				     cback => [sub {$self->enumshares}]);
  $app->{smb} = $self;
  $self->{tree} = $app->{network}->{tree};
  $self->{node} = $app->{network}->{smbnode};
  return $self;
}

our $helpmessage = <<EOF;
The SMB modules allows you to discover windows networks
on any subnet seperate from any WINS servers or NetBIOS
broadcasting.  It achieves this by send a node status
query to each IP you specify.

It will also suffix three identifiers onto the
 //workgroup/name path depedning on the host:
  LM  - Local Master
  DMB - Domain Master Browser
  DC  - Domain Controller

It also allows you to enumerate the shares on any SMB
host as any username/password login.
EOF

sub help {
  my $app  = shift || return undef;
  my $self = shift || return undef;
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  my $text = new Gtk::Text(undef, undef);
  my $scroll = new Gtk::ScrolledWindow($text->hadj, $text->vadj);
  $text->set_editable(0);
  $text->insert(undef, undef, undef, $helpmessage);
  $scroll->add($text);
  $scroll->set_policy(1,1);
  $close->signal_connect('clicked', sub {$win->destroy});
  $win->set_title('SMB Help');
  $win->set_default_size($app->{defsize}->[0]*2/3, $app->{defsize}->[1]*2/3);
  $win->position(1);
  $win->set_transient_for($app->{toplevel});
  $win->set_modal(1);
  $win->vbox->pack_start($scroll, 1, 1, 1);
  $win->action_area->add($close);
  $win->show_all;
  return 1;
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, keyattr => undef, rootname => 'smb');
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray =>
				  ['master_browser', 'controllers', 'host', 'share', 'workgroup'])});
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {};
  @{$data->{workgroup}} = map {$_->serial_data} $self->children;
  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  if (ref $data->{workgroup} eq 'ARRAY') {
    for (@{$data->{workgroup}}) {
      my $group = netmap::smb::workgroup->new;
      my $node = $self->{tree}->insert
	($self->{node}, undef, [$_->{name}], 1, undef, undef, undef, undef, 0, 1);
      $group->{addcback} = [\&addcback, $self];
      $group->{delcback} = [\&delcback, $self];
      $group->node($node);
      push @{$self->{children}}, [$_->{name}, $group, $node];

      $group->deserial_data($_);
    }
  }
  return 1;
}

sub scansubnet {
  my $app = shift || return undef;
  my ($type, $number, $obj, $self) = @_;
  return undef if (ref $self ne 'netmap::smb');

  for my $host ($obj->hosts) {
    queryhost($app, 'host', $host->ipv4_address, $host, $self);
  }
  for my $subnet ($obj->subnets) {
    scansubnet($app, 'subnet', $subnet->number, $subnet, $self);
  }

  return 1;
}

sub queryhost {
  my $app = shift || return undef;
  my ($type, $ip, $obj, $self) = @_;
  return undef if (ref $self ne 'netmap::smb');
  my $host = netmap::smb::host->new(ipv4_address => $ip, scan => 1, autoscan => 0) || return undef;
  $self->add($host->path, $host);
  $self->{app}->{network}->_runUpdateHooks
    if ((defined $app->{network}->curobj) &&
	($app->{network}->curtype eq 'host') &&
	($app->{network}->curobj->ipv4_address == $ip));
}

sub findbyname {
  my $self = shift || return undef;
  my $name = shift || return undef;
  for my $group ($self->children) {
    for my $host ($group->children) {
      return $host if (lc $host->name eq lc $name);
    }
  }
  return undef;
}

sub findbyip {
  my $self = shift || return undef;
  my $ip   = shift || return undef;
  for my $group ($self->children) {
    for my $host ($group->children) {
      return $host if ($host->ipv4_address eq $ip);
    }
  }
  return undef;
}

sub children {
  my $self = shift || return undef;
  my @ret = map {$_->[1]} @{$self->{children}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub find {
  my $self = shift || return undef;
  my $name = shift || return undef;
  my $obj;
  if ($obj = $self->findgroup($name)) {
    return ['smbworkgroup', $obj];
  } else {
    local $_;
    for (map {$_->[1]} @{$self->{children}}) {
      next unless (defined $_);
      if ($obj = $_->find($name)) {
	return ['smbhost', $obj];
      }
    }
  }
  return undef;
}

sub findgroup {
  my $self = shift || return undef;
  my $name = shift || return undef;
  my ($obj) = map {$_->[1]} grep {$_->[0] eq $name} @{$self->{children}};
  return $obj;
}

sub add {
  my $self = shift || return undef;
  my $path = shift || return undef;
  my $obj  = shift || return undef;
  $path =~ s/^\\\\//;
  my @path = split /\\/, $path;
  my $wgroup = shift @path;
  my $hname  = shift @path;
  unless (defined $self->findgroup($wgroup)) {
    my $g = netmap::smb::workgroup->new(name => $wgroup);
    $g->{addcback} = [\&addcback, $self];
    $g->{delcback} = [\&delcback, $self];
    my $node = $self->{tree}->insert
      ($self->{node}, undef, [$wgroup], 1, undef, undef, undef, undef, 0, 1)
	|| return undef;
    $g->node($node);
    push @{$self->{children}}, [$wgroup, $g, $node];
  }
  if (defined $hname) {
    my ($it) = grep {$_->[0] eq $wgroup} @{$self->{children}};
    $it->[1]->add($obj);
#    my $iphost = $self->{app}->{network}->find('host', $obj->ipv4_address);
#    $iphost->{issmb} = 1 if (defined $iphost);
  }
  return 1;
}

sub addcback {
  my ($group, $host, $self) = @_;
  $host->node
    ($self->{tree}->insert
     ($host->parent->node, undef, [$host->name], 1, undef, undef, undef, undef, 0, 1));

  my $iphost = $self->{app}->{network}->find('host', $host->ipv4_address);
  $iphost->{issmb} = 1 if (defined $iphost);

  return 1;
}

sub del {
  my $self = shift || return undef;
  my $name = shift || return undef;
  my $g = $self->findgroup($name);
  $g->{addcback} = undef;
  $g->{delcback} = undef;
  $self->{app}->{network}->{tree}->remove($g->node) if (defined $g->node);
  $g->{node} = undef;
  local $_;
  for ($g->children) {
    $_->{node} = undef;
  }
  @{$self->{children}} = grep {$_->[0] ne $name} @{$self->{children}};
  return 1;
}

sub delcback {
  my ($group, $host, $self) = @_;
  $self->{app}->{network}->{tree}->remove($host->node) if (defined $host->node);
  return 1;
}

1;
