# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::smb::host;
use Net::NBName;
use XML::Simple;
use Filesys::SmbClient;
use strict;

our $debug = 0;
our $AUTOLOAD;
our %fields =
  (name                   => 1,
   autoscan               => 1,
   node                   => 1,
   elec                   => 1,
   ipv4_address           => 1,
   domain_master_browser  => 1,
   master_browser         => 1,
   domain_controller      => 1,
   domain                 => 1,
   username               => 1,
   password               => 1,
   parent                 => 1,
   msg                    => []
  );

our @share_types =
  (undef,
   'WORKGROUP',
   'SERVER',
   'FILE_SHARE',
   'PRINTER_SHARE',
   'COMMS_SHARE',
   'IPC_SHARE',
   'DIR',
   'FILE',
   'LINK');

sub as_string {
  my $self = shift || return undef;
  my $ret = sprintf "\\\\%s\\%s (%s)\n", $self->domain, $self->name, $self->ipv4_address;
  for ($self->shares) {
    $ret .= sprintf "\\\\%s\\%s\\%s (%s)\n", $self->domain, $self->name, $_->[1], $share_types[$_->[0]];
  }
  return $ret;
}

sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $self  = {shares => [], msg => [], autoscan => 0};
  bless $self, $class;
  $self->{nb} = new Net::NBName;
  my %param = @_;
  local $_;
  for (keys %param) {
    $self->$_($param{$_});
  }
  if ($param{scan}) {
    $self->query_status || return undef;
  }
  return $self;
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, keyattr => undef, rootname => 'host');
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray => ['share'])});
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {shares => []};
  for (keys %fields) {
    next if (($_ eq 'parent') || ($_ eq 'autoscan') || ($_ eq 'node'));
    my $v = $self->$_;
    $data->{$_} = $v if (defined $v);
  }

  for (@{$self->{shares}}) {
    push @{$data->{share}}, {type    => $share_types[$_->[0]],
			     name    => $_->[1],
			     comment => $_->[2]};
  }
  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  (ref $data eq 'HASH') || return undef;

  for (@{$data->{share}}) {
    push @{$self->{shares}}, [indexof($_->{type}, @share_types), $_->{name}, $_->{comment}];
  }

  delete $data->{share};
  for (keys %$data) {
    $self->$_($data->{$_});
  }
  return 1;
}

sub indexof {
  my $it = shift || return undef;
  my $i=0;
  for (@_) {
    return $i if($_ eq $it);
    $i++;
  }
  return undef;
}

sub path {
  my $self = shift || return undef;
  return '\\\\' . $self->domain . '\\' . $self->name;
}

sub shares {
  my $self = shift || return undef;
  if (($#{$self->{shares}} == -1) && ($self->{autoscan})) {
    $self->scan_shares || return undef;
  }
  my @ret = @{$self->{shares}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub query_status {
  my $self = shift || return undef;
  return undef unless (defined $self->ipv4_address);
  my $ns = $self->{nb}->node_status($self->ipv4_address) || return undef;
  for my $rr ($ns->names) {
    if ($rr->suffix == 0x00) {
      if    ($rr->G eq 'UNIQUE') {$self->name($rr->name)}
      elsif ($rr->G eq 'GROUP')  {$self->domain($rr->name)}
    } elsif ($rr->suffix == 0x03 && $rr->G eq 'UNIQUE') {
      $self->msg($rr->name);
    } elsif ($rr->suffix == 0x1B && $rr->G eq 'UNIQUE') {
      $self->domain_master_browser(1);
      $self->parent->domain_master_browser($rr->name) if (defined $self->parent);
    } elsif ($rr->suffix == 0x1C && $rr->G eq 'GROUP') {
      $self->domain_controller(1);
      $self->parent->controllers->($rr->name) if (defined $self->parent);
    } elsif ($rr->suffix == 0x1D && $rr->G eq 'UNIQUE') {
      $self->master_browser(1);
      $self->parent->master_browser->($rr->name) if (defined $self->parent);
    } elsif ($rr->suffix == 0x1E && $rr->G eq 'GROUP') {
      $self->elec($rr->name);
    } elsif ($rr->suffix == 0x20 && $rr->G eq 'UNIQUE') {
      $self->server($rr->name);
    }
  }
  return 1;
}

sub scan_shares {
  my $self = shift || return undef;
  my $smb  = new Filesys::SmbClient(username  => $self->username,
				    password  => $self->password,
				    workgroup => $self->domain,
				    debug     => $debug)
    || return undef;
  return undef unless (defined $self->name);
  my $in = $smb->opendir('smb://' . $self->name) || return undef;
  @{$self->{shares}} = $smb->readdir_struct($in);
  $smb->closedir($in);
  return 1;
}

sub AUTOLOAD {
  my $self  = shift || return undef;
  my @a     = split '::', $AUTOLOAD;
  my $field = $a[-1];
  return __field($self, $field, @_);
}

sub __field {
  my $self  = shift || return undef;
  my $field = shift || return undef;
  my $value = shift;
  my $ret   = undef;
  if (defined $fields{$field}) {
    $ret = $self->{$field};
    if (ref $fields{$field} eq 'ARRAY') {
      if (wantarray) {return @{$self->{$field}}}
      else           {return [@{$self->{$field}}]}
    } elsif (defined $value) {
      if (ref $fields{$field} eq 'CODE') {
	if (&{$fields{$field}}($value)) {
	  $self->{$field} = $value;
	} else {
	  $ret = undef;
	}
      } elsif (ref $fields{$field} eq 'ARRAY') {
	push @{$self->{$field}}, $value;
      } else {
	$self->{$field} = $value;
      }
    }
  }

  return $ret;
}

1;
