# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::smb::workgroup;
use strict;
use XML::Simple;

our $AUTOLOAD;
our %fields =
  (name                   => 1,
   node                   => 1,
   domain_master_browser  => 1,
   master_browser         => [],
   controllers            => []
  );

sub as_string {
  my $self = shift || return undef;
  return sprintf "\\\\%s\n", $self->name;
}

sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $self =
    {children       => [],
     master_browser => [],
     stat_on_add    => 1,
     controllers    => []};
  bless $self, $class;
  my %param = @_;
  local $_;
  for (keys %param) {
    $self->$_($param{$_});
  }
  return $self;
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, keyattr => undef, rootname => 'workgroup');
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray => ['master_browser', 'controllers', 'host', 'share'])});
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {};
  @{$data->{host}} = map {$_->serial_data} $self->children;
  for (keys %fields) {
    next if ($_ eq 'node');
    my $v = $self->$_;
    $data->{$_} = $v if (defined $v);
  }
  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  (ref $data eq 'HASH') || return undef;
  my @host = @{$data->{host}} if (ref $data->{host} eq 'ARRAY');
  delete $data->{host};
  local $_;
  for (keys %$data) {
    $self->$_($data->{$_});
  }
  for (@host) {
    my $host = netmap::smb::host->new;
    $host->deserial_data($_);
    $self->add($host);
  }
  return 1;
}

sub stat_on_add {
  my $self = shift || return undef;
  $self->{stat_on_add} = $self->{stat_on_add} ? 0 : 1;
  return 1;
}

sub add {
  my $self  = shift || return undef;
  my $child = shift || return undef;
  if (($self->{stat_on_add}) && !($child->query_status)) {
    $child->{parent} = undef;
    return undef;
  }
  return undef unless (defined $child->name);
  return undef if (grep {$_->[0] eq $child->name} @{$self->{children}});
  $child->parent($self);

  push @{$self->{children}}, [$child->name, $child];

  if (ref $self->{addcback} eq 'ARRAY') {
    my @c = @{$self->{addcback}};
    my $code = shift @c;
    if (ref $code eq 'CODE') {
      &$code($self, $child, @c);
    }
  }

  return 1;
}

sub refresh {
  my $self = shift || return undef;
  @{$self->{children}} = map {[$_->[1]->name, $_->[1]]} @{$self->{children}};
  return 1;
}

sub find {
  my $self = shift || return undef;
  my $name = shift || return undef;
  my ($ret) = map {$_->[1]} grep {$_->[0] eq $name} @{$self->{children}};
  return $ret;
}

sub del {
  my $self = shift || return undef;
  my $name = shift || return undef;
  my ($child) = map {$_->[1]} grep {$_->[0] eq $name} @{$self->{children}};
  if (defined $child) {
    @{$self->{children}} = grep {$_->[0] ne $name} @{$self->{children}};
    $child->{parent} = undef;
    if (ref $self->{delcback} eq 'ARRAY') {
      my @c = @{$self->{delcback}};
      my $code = shift @c;
      if (ref $code eq 'CODE') {
	&$code($self, $child, @c);
      }
    }
  }
  return 1;
}

sub children {
  my $self = shift || return undef;
  my @ret = map {$_->[1]} @{$self->{children}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub AUTOLOAD {
  my $self  = shift || return undef;
  my @a     = split '::', $AUTOLOAD;
  my $field = $a[-1];
  return __field($self, $field, @_);
}

sub __field {
  my $self  = shift || return undef;
  my $field = shift || return undef;
  my $value = shift;
  my $ret   = undef;
  if (defined $fields{$field}) {
    $ret = $self->{$field};
    if (ref $fields{$field} eq 'ARRAY') {
      if (wantarray) {return @{$self->{$field}}}
      else           {return [@{$self->{$field}}]}
    } elsif (defined $value) {
      if (ref $fields{$field} eq 'CODE') {
	if (&{$fields{$field}}($value)) {
	  $self->{$field} = $value;
	} else {
	  $ret = undef;
	}
      } elsif (ref $fields{$field} eq 'ARRAY') {
	push @{$self->{$field}}, $value;
      } else {
	$self->{$field} = $value;
      }
    }
  }

  return $ret;
}

1;
