# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::subnet;
use strict;
use Net::IPv4Addr qw(:all);
use XML::Simple;
use Gtk;
use netmap::host;

our $AUTOLOAD;
our %fields =
  ('number' => \&valid_network,
   'name'   => 1);

sub findsubnetfor {
  my $self = shift || return undef;
  my $ip   = shift || return undef;
  my $it   = undef;
  for my $subnet ($self->subnets) {
    if (ipv4_in_network($subnet->number, $ip)) {
      my $next = $subnet->findsubnetfor($ip);
      $it = (defined $next) ? $next : $subnet;
      last;
    }
  }
  return $it;
}

sub elemof {
  my $it = shift || return undef;
  local $_;
  for (@_) {return 1 if ($_ eq $it)}
  return undef;
}

sub new {
  my $proto = shift || return undef;
  my $class = (ref $proto) || $proto;
  my %param  = @_;
  my $self  =
    {#okclass => ['netmap::host', 'netmap::subnet'],
     hostclass => 'netmap::host',
     subnetclass => 'netmap::subnet',
     children => []};
  bless $self, $class;
  local $_;
  for (keys %param) {
    next unless(defined $fields{$_});
    $self->$_($param{$_});
  }
  return $self;
}

sub serialize {
  my $self = shift || return undef;
  my $xml = XMLout($self->serial_data, rootname => 'subnet', keyattr => undef);
  return $xml;
}

sub deserialize {
  my $self = shift || return undef;
  my $xml  = shift || return undef;
  $self->deserial_data(eval{XMLin($xml, keyattr => undef, forcearray => ['port'])});
}

sub serial_data {
  my $self = shift || return undef;
  my $data = {number => $self->number};
  local $_;
  for (keys %fields) {
    next if($_ eq 'number');
    my $v = $self->$_;
    $data->{$_} = $v if (defined $v);
  }

  @{$data->{subnet}} = map {$_->serial_data} $self->subnets;
  @{$data->{host}}   = map {$_->serial_data} $self->hosts;

  return $data;
}

sub deserial_data {
  my $self = shift || return undef;
  my $data = shift || return undef;
  $self->number($data->{number});
  my @subnet = @{$data->{subnet}} if (ref $data->{subnet} eq 'ARRAY');
  my @host   = @{$data->{host}}   if (ref $data->{host}   eq 'ARRAY');
  delete $data->{number};
  delete $data->{subnet};
  delete $data->{host};
  local $_;
  for (keys %$data) {
    $self->$_($data->{$_});
  }
  for (sort {subnetcmp($a->{number}, $b->{number})} @subnet) {
    $self->_rebuild_subnet($_);
  }
  for (sort {ipv4cmp($a->{ipv4}->{address}, $b->{ipv4}->{address})} @host) {
    $self->_rebuild_host($_);
  }
  return 1;
}

sub subnetcmp {
  my ($a, $b) = @_;
  my @a = $a =~ /(\d+)(\.\d+){3}\s*\/\s*(\d+)/;
  my @b = $b =~ /(\d+)(\.\d+){3}\s*\/\s*(\d+)/;
  return
    $a[0] <=> $b[0] ||
    $a[1] <=> $b[1] ||
    $a[2] <=> $b[2] ||
    $a[3] <=> $b[3] ||
    $b[4] <=> $a[4];
}

sub ipv4cmp {
  my ($a, $b) = @_;
  my @a = split /\./, $a, 4;
  my @b = split /\./, $b, 4;
  return
    $a[0] <=> $b[0] ||
    $a[1] <=> $b[1] ||
    $a[2] <=> $b[2] ||
    $a[3] <=> $b[3];
}

sub _rebuild_subnet {
  my $self = shift || return undef;
  my $data = shift || return undef;
  my $subnet = new netmap::subnet;
  $subnet->deserial_data($data);
  $self->add($subnet);
}

sub _rebuild_host {
  my $self = shift || return undef;
  my $data = shift || return undef;
  my $host = new netmap::host;
  $host->deserial_data($data);
  $self->add($host);
}

sub hosts {
  my $self = shift || return undef;
  my @ret = map {$_->[1]} grep {ref $_->[1] eq $self->{hostclass}} @{$self->{children}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub subnets {
  my $self = shift || return undef;
  my @ret = map {$_->[1]} grep {ref $_->[1] eq $self->{subnetclass}} @{$self->{children}};
  if (wantarray) {return @ret}
  else           {return [@ret]}
}

sub get {
  my $self = shift || return undef;
  my $this = shift || return undef;
  $self->_refresh;
  my ($ret) = map {$_->[1]} grep {$_->[0] eq $this} @{$self->{children}};
  return $ret;
}

sub _nameof {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  my $ref  = ref $obj;
  if (defined $ref) {
    if ($ref eq $self->{hostclass}) {
      $obj = $obj->ipv4_address;
    } elsif ($ref eq $self->{subnetclass}) {
      $obj = $obj->number;
    } else {
      return undef;
    }
  }
  return $obj;
}

sub del {
  my $self = shift || return undef;
  my $this = shift || return undef;
  $this = $self->_nameof($this) || return undef;
  $self->_refresh;
  @{$self->{children}} = grep {$_->[0] ne $this} @{$self->{children}};
  return 1;
}

sub add {
  my $self = shift || return undef;
  return undef unless(defined $self->number);
  local $_;
  my $i=0;
  for (@_) {
    $i++ if ($self->_add($_));
  }
  return $i;
}

sub _add {
  my $self = shift || return undef;
  my $obj  = shift || return undef;
  my $ref  = ref $obj;
  my $this = undef;
  if ($ref eq $self->{hostclass}) {
    $this = $obj->ipv4_address;
  } elsif ($ref eq $self->{subnetclass}) {
    $this = $obj->number;
  } else {
    return undef;
  }
  if (defined $this) {
    $self->_refresh;
    if (grep {$_->[0] eq $this} @{$self->{children}}) {
      return undef;
    } else {
      if (ipv4_in_network($self->number, $this)) {
	$obj->{_parent} = $self;
	push @{$self->{children}}, [$this, $obj];
	return 1;
      } else {
	return undef;
      }
    }
  } else {
    return undef;
  }
}

sub _refresh {
  my $self = shift || return undef;
  local $_;
  for (@{$self->{children}}) {
    my $ref = ref $_->[1];
    my $this = undef;
    if ($ref eq $self->{hostclass}) {
      $this = $_->[1]->ipv4_address;
    } elsif ($ref eq $self->{subnetclass}) {
      $this = $_->[1]->number;
    } else {
      $this = undef;
    }
    $_->[0] = $this;
  }
  @{$self->{children}} = grep {defined $_->[0]} @{$self->{children}};
  return 1;
}

sub valid_network {
  local $_ = shift || return undef;
  my ($net,$msk) = eval{ipv4_network($_)};
  if ((!defined $net) || (!defined $msk)) {
    warn "netmap::host::valid_network: Error parsing $_\n";
    return undef;
  } else {
    if (/$net\s*\/\s*$msk/) {
      return 1;
    } else {
      return 0;
    }
  }
}

sub AUTOLOAD {
  my $self  = shift || return undef;
  my @a     = split '::', $AUTOLOAD;
  my $field = $a[-1];
  return __field($self, $field, @_);
}

sub __field {
  my $self  = shift || return undef;
  my $field = shift || return undef;
  my $value = shift;
  my $ret   = undef;
  if (defined $fields{$field}) {
    $ret = $self->{$field};
    if (defined $value) {
      if (ref $fields{$field} eq 'CODE') {
	if (&{$fields{$field}}($value)) {
	  $self->{$field} = $value;
	} else {
	  $ret = undef;
	}
      } else {
	$self->{$field} = $value;
      }
    }
  }
  return $ret;
}

1;
